#ifndef APP_APP_VIEW_H
#define APP_APP_VIEW_H

#ifdef _MSC_VER
#pragma once
#endif

#include "site.h"
#include "view.h"

#include <windows.h>
#include <memory>

/*************************************************

	AvP[V r[NX

**************************************************/

APP_BEGIN_NAMESPACE

class AppDoc;
class Command;
class Image;
struct FPOINT;
union PA_COLOR;

class AppView : public View
{
public:
	AppView();
	virtual ~AppView() throw();

	virtual void OnInitialUpdate();
	virtual LRESULT WindowProc(UINT msg,WPARAM wparam,LPARAM lparam);
	virtual bool OnIdle();
protected:
	virtual void OnCommand(int id,int notifyCode,HWND hwnd);

	AppDoc *GetDocument() const {
		return reinterpret_cast<AppDoc*>(View::GetDocument());
	}

	void OnLButtonDown(int x,int y,int keyFlags);
	void OnLButtonUp(int x,int y,int keyFlags);
	void OnLButtonDblClk(int x,int y,int keyFlags);
	void OnRButtonDown(int x,int y,int keyFlags);
	void OnMouseMove(int x,int y,int keyFlags);
	void OnKeyDown(int key,UINT keyData);
	void OnKeyUp(int key,UINT keyData);
	void OnCaptureChanged(HWND hwnd);
//	bool OnSetCursor(HWND hwnd,int hitTest,int mouseMessage);
	void OnSize(UINT type,int width,int height);
	virtual void OnDestroy();
	void OnActivateApp(BOOL isActivate);

//	bool SetCursor();
	void CalcDocRect(RECT *rect) const;
	POINT GetDocPoint(int x,int y) const;
	POINT GetMessagePoint() const;
	void CallTool(int event,POINT point,int key);

	void OnViewZoom(int id);
	void SetZoom(int zoom);
	void OnUpdateViewZoom(int id,Command *command);
	void OnViewZoomOut();
	void OnViewZoomIn();

	void OnViewMat(int id);
	void OnUpdateViewMat(int id,Command *command);

	void SetScrollSize(SIZE docSize);
	POINT ViewMargin() const;
	void ImageToView(RECT *viewRect,const RECT &imageRect) const;
	void ViewToImage(RECT *imageRect,const RECT &viewRect) const;
	void DocToImage(FPOINT *imagePoint,POINT docPoint) const;

	virtual void OnUpdateCommand(int id,Command *command);

	virtual void SetWindowTitle();
	void SetStatusText();

	void OnPaint();
	void Display(HDC hdc);
	void Matting(Image *dest,const Image *src
		,POINT imageOrigin,POINT viewOrigin);
	void SizingAndMatting(Image *dest,const Image *src
		,POINT imageOrigin,POINT viewOrigin,int zoom);

	void OnTurn();
	void Pause(int reason,bool paused);
	bool IsPaused() const;
	void OnUserPause();
	void OnUpdateUserPause(Command *command);
	void OnEraseAll();
	void OnUpdateEraseAll(Command *command);

	void OnChangeOptions();
	void OnOptions();

	virtual void OnFileClose();

	void OnSelectTool(int id);
	void OnUpdateSelectTool(int id,Command *command);

	unsigned long OnGetPixel(int x,int y);

	void OnEditUndo();
	void OnUpdateEditUndo(Command *command);
	void OnEditRedo();
	void OnUpdateEditRedo(Command *command);
	void OnEditClear();
private:
	const std::auto_ptr<class AppViewMember> m_m;
};

APP_END_NAMESPACE

#endif

