#ifndef APP_BUFFER_HOLDER_H
#define APP_BUFFER_HOLDER_H

#ifdef _MSC_VER
#pragma once
#endif

#include "site.h"

#include "exception.h"
#include "error.h"
#include "verify.h"

#include <assert.h>
#include <stdlib.h>
#include <memory.h>

/*************************************************

	QƃJEgɂobt@[ێ

**************************************************/

APP_BEGIN_NAMESPACE

#ifndef NDEBUG
#define TESTCODE(a_index) (unsigned char)((0xcefaadab >> (8* (a_index%4))) & 0xff)
#define TEST_BLOCK_SIZE 256
#endif

class BufferHolder
{
public:
	BufferHolder()
		:m_ptr(NULL)
		,m_ref(1)
		,m_bytes(0)
	{
	}

	BufferHolder(const BufferHolder &source,int bytes=0)
		:m_ptr(NULL)
		,m_ref(1)
		,m_bytes(0)
	{
		assert(bytes>=0);

		int allocBytes=bytes;
		if(allocBytes==0) {
			allocBytes=source.Bytes();
		}
		Alloc(allocBytes);
		memcpy(Get(),source.Get(),allocBytes);
	}

	~BufferHolder() throw()
	{
		Free();
	}

	//
	void Free() throw()
	{
		if(m_ptr!=NULL) {
#ifndef NDEBUG
			OverFlowTest();
#endif
			unsigned char *realPtr=GetRealPtr();
			free(realPtr);
			m_ptr=NULL;
			m_bytes=0;
		}
	}

	//m
	void Alloc(int bytes)
	{
		assert(bytes>=0);

		Free();
		int wholeBytes=bytes;
#ifndef NDEBUG
		wholeBytes+=TEST_BLOCK_SIZE*2;
#endif
		void *newPtr=malloc(wholeBytes);
		if(newPtr==NULL) {
			throw Exception(APPERR_OUT_OF_MEMORY);
		}
		SetRealPtr(newPtr);
		m_bytes=bytes;

#ifndef NDEBUG
		SetTestBlock();
#endif
	}

	//AhX𓾂
	unsigned char *Get()	//ŨAhXƂOɂ
	{
		return m_ptr;
	}

	const unsigned char *Get() const
	{
		return m_ptr;
	}

	//傫
	int Bytes() const
	{
		return m_bytes;
	}

	//vfύX
	void Realloc(int bytes)
	{
		assert(bytes>=0);

		if(m_ptr==NULL) {
			Alloc(bytes);
		} else {
			int allocBytes=bytes;
#ifndef NDEBUG
			allocBytes+=TEST_BLOCK_SIZE*2;
#endif
			unsigned char *realPtr=GetRealPtr();
			void *newPtr=realloc(realPtr,allocBytes);
			if(newPtr==NULL) {
				throw Exception(APPERR_OUT_OF_MEMORY);
			}
			SetRealPtr(newPtr);
			m_bytes=bytes;
#ifndef NDEBUG
			SetTestBlock();
#endif
		}
	}

	//QƃJEg炷
	void IncreaseReferenceCount()
	{
		assert(m_ref>=0);
		++m_ref;
	}

	//QƃJEg炷
	void DecreaseReferenceCount()
	{
		assert(m_ref>=1);
		--m_ref;
		if(m_ref==0) {
			delete this;
		}
	}

	//LĂ邩
	bool IsSharing() const
	{
		return m_ref>1;
	}

protected:

#ifndef NDEBUG
	void OverFlowTest()
	{
		const unsigned char *ptr=GetRealPtr();
		FOR(int i=0;i<TEST_BLOCK_SIZE;++i) {
			assert(*ptr == TESTCODE(i) );
			++ptr;
		}

		ptr=GetRealPtr()+Bytes()+TEST_BLOCK_SIZE;
		FOR(int i=0;i<TEST_BLOCK_SIZE;++i) {
			assert(*ptr == TESTCODE(i) );
			++ptr;
		}
	}

	void SetTestBlock()
	{
		unsigned char *ptr=GetRealPtr();
		FOR(int i=0;i<TEST_BLOCK_SIZE;++i) {
			*ptr = TESTCODE(i);
			++ptr;
		}

		ptr=GetRealPtr()+Bytes()+TEST_BLOCK_SIZE;
		FOR(int i=0;i<TEST_BLOCK_SIZE;++i) {
			*ptr = TESTCODE(i);
			++ptr;
		}
	}
#endif

private:
	unsigned char *GetRealPtr()
	{
		unsigned char *ptr=m_ptr;
#ifndef NDEBUG
		ptr-=TEST_BLOCK_SIZE;
#endif
		return ptr;
	}

	void SetRealPtr(void *newPtr)
	{
		unsigned char *ptr=reinterpret_cast<unsigned char*>(newPtr);
#ifndef NDEBUG
		ptr+=TEST_BLOCK_SIZE;
#endif
		m_ptr=ptr;
	}
private:
	//`Ȃ
	const BufferHolder &operator=(const BufferHolder &source);

	unsigned char *m_ptr;	//AhX
	int m_bytes;	//oCg
	int m_ref;	//QƃJEg
};

APP_END_NAMESPACE

#endif
