#include "prefix.h"

#include "colorpanel.h"

#include "updatecontrol.h"
#include "volumecontrol.h"
#include "colortablecontrol.h"
#include "curcolorcontrol.h"
#include "color.h"
#include "tool.h"
#include "channel.h"
#include "winman.h"
#include "winmessage.h"
#include "registry.h"
#include "gdiobject.h"
#include "idtovalue.h"

#include "colordetail.h"

#include "resdialog.h"
#include "resbitmap.h"

#include <windows.h>
#include <commctrl.h>
#include <limits.h>
#include <tchar.h>

/*************************************************

	F̃pl

**************************************************/

APP_BEGIN_NAMESPACE

static const IDToValueTable s_buttons[]={
	{IDC_COLORING_SINGLE,IDB_COLORING_SINGLE},
	{IDC_COLORING_GRADATION,IDB_COLORING_GRADATION},
	{IDC_COLORING_CYCLE,IDB_COLORING_CYCLE},
	{IDC_COLORING_RANDOM,IDB_COLORING_RANDOM},
	{IDC_DETAIL_COLOR,IDB_DETAIL},
};
#define BUTTON_COUNT (sizeof(s_buttons)/sizeof(IDToValueTable))

#define REGISTRYKEY "Panel\\Color"

class ColorPanelMember
{
public:
	ColorTableControl colorTableControl;
	VolumeControl foreOpacityControl;
	VolumeControl backOpacityControl;
	CurrentColorControl currentColorControl;

	GdiObject buttonImage[BUTTON_COUNT];

	int level;
};


ColorPanel::ColorPanel()
:Panel(_T(REGISTRYKEY),IDD_COLOR)
,m_m(new ColorPanelMember)
{
}

ColorPanel::~ColorPanel() throw()
{

}

//WXgƌ
void ColorPanel::ExchangeSettings(Registry *registry)
{
	Panel::ExchangeSettings(registry);

	registry->OpenSubKey(_T(REGISTRYKEY));
	registry->Exchange(_T("Level"),&m_m->level,0);
	if(registry->IsReading()) {
		if(m_m->level<0) {
			m_m->level=0;
		}
		if(m_m->level>UCHAR_MAX) {
			m_m->level=UCHAR_MAX;
		}
	}
}


//
void ColorPanel::OnInitDialog()
{
	Panel::OnInitDialog();

	SetButtonImage(m_m->buttonImage,s_buttons,BUTTON_COUNT);

	m_m->colorTableControl.SubclassDialogItem(
		IDC_COLOR_TABLE,this);
	m_m->colorTableControl.AllocBuffer();
	m_m->colorTableControl.SetLevel(m_m->level);

	m_m->foreOpacityControl.SubclassDialogItem(
		IDC_COLOR_FORE_OPACITY,this);
	m_m->backOpacityControl.SubclassDialogItem(
		IDC_COLOR_BACK_OPACITY,this);

	m_m->currentColorControl.SubclassDialogItem(
		IDC_COLOR_CURRENT,this);

	HWND levelSlider(GetDlgItem(IDC_COLOR_LEVEL));
	::SendMessage(levelSlider,TBM_SETRANGE,TRUE,MAKELONG(0,255));

	m_m->foreOpacityControl.SetRange(0,255);
	m_m->foreOpacityControl.SetShowMode(VOLUME_SHOW_INTEGER);

	m_m->backOpacityControl.SetRange(0,255);
	m_m->backOpacityControl.SetShowMode(VOLUME_SHOW_INTEGER);
}

LRESULT ColorPanel::WindowProc(UINT msg,WPARAM wparam,LPARAM lparam)
{
	switch(msg) {
	case APP_WM_SET_TOOL:
		OnSetTool(wparam);
		break;

	case WM_VSCROLL:
		OnVScroll((int)LOWORD(wparam),(short int)HIWORD(wparam),(HWND)lparam);
		break;

	default:
		return Panel::WindowProc(msg,wparam,lparam);
	}
	return 0;
}

void ColorPanel::OnCommand(int id,int notifyCode,HWND hwnd)
{
	switch(id) {
	case IDC_COLORING_SINGLE:
	case IDC_COLORING_GRADATION:
	case IDC_COLORING_CYCLE:
	case IDC_COLORING_RANDOM:
		if(notifyCode==BN_CLICKED) {
			OnColoring();
		}
		break;

	case IDC_DETAIL_COLOR:
		if(notifyCode==BN_CLICKED) {
			OnColorDetail();
		}
		break;

	default:
		Panel::OnCommand(id,notifyCode,hwnd);
		break;
	}
}

void ColorPanel::OnControl(int id,int code,LPARAM lparam)
{
	switch(id) {
	case IDC_COLOR_FORE_OPACITY:
	case IDC_COLOR_BACK_OPACITY:
		if(code==VOLUME_CHANGED) {
			UpdateData(UPDATE_VAR);
		}
		break;
	
	case IDC_COLOR_TABLE:
		if(code==COLORTABLE_SELECT) {
			OnSelectColor(lparam);
		}
		break;

	default:
		Panel::OnControl(id,code,lparam);
		break;
	}
}

static const IDToValueTable s_coloringTable[]=
{
	{IDC_COLORING_SINGLE,COLORING_SINGLE},
	{IDC_COLORING_GRADATION,COLORING_GRADATION},
	{IDC_COLORING_CYCLE,COLORING_CYCLE},
	{IDC_COLORING_RANDOM,COLORING_RANDOM},

	{0,0}
};

//Rg[ƕϐ̃f[^\̍XV
void ColorPanel::DataExchange(UpdateControl *updater)
{
	COLOR_INFO colorInfo;
	GetColorInfo(&colorInfo);

	FOR(int i=0;i<COLOR_COUNT;++i) {
		PA_COLOR curColor;
		curColor=colorInfo.color[i];
		int opacity=curColor.channel[CHANNEL_OPACITY];

		int controlID=IDC_COLOR_FORE_OPACITY;
		if(i==BACKCOLOR) {
			controlID=IDC_COLOR_BACK_OPACITY;
		}

		updater->Volume(controlID,&opacity);
		curColor.channel[CHANNEL_OPACITY]=opacity;

		colorInfo.color[i]=curColor;
	}

	updater->Slider(IDC_COLOR_LEVEL,&m_m->level);
	updater->RadioButton(s_coloringTable,&(colorInfo.coloring));

	bool useColor=ToolType(GetTool())->useColor;
	updater->Enable(IDC_COLOR_CURRENT,useColor);

	bool useBackOpacity=(useColor || colorInfo.coloring!=COLORING_SINGLE);
	updater->Enable(IDC_COLOR_BACK_OPACITY,useBackOpacity);

	if(!updater->ToControl()) {
		SetColorInfo(colorInfo);
	}
}

//FI
void ColorPanel::OnSelectColor(int color)
{
	COLOR_INFO colorInfo;
	GetColorInfo(&colorInfo);

	int selection=FORECOLOR;
	if(!colorInfo.isForeSelected) {
		selection=BACKCOLOR;
	}
	PA_COLOR *curColor=&(colorInfo.color[selection]);

	curColor->channel[CHANNEL_RED]=GetRValue(color);
	curColor->channel[CHANNEL_GREEN]=GetGValue(color);
	curColor->channel[CHANNEL_BLUE]=GetBValue(color);

	SetColorInfo(colorInfo);

	m_m->currentColorControl.InvalidateRect();
}

//Fe[ủ̃XC_[𓮂
void ColorPanel::OnVScroll(int,short int,HWND)
{
	UpdateData(UPDATE_VAR);
	m_m->colorTableControl.SetLevel(m_m->level);
}

//c[؂ւ
void ColorPanel::OnSetTool(int)
{
	UpdateData(UPDATE_CONTROL);
	m_m->currentColorControl.InvalidateRect();
}

//F̕ω̃{^
void ColorPanel::OnColoring()
{
	UpdateData(UPDATE_VAR);
	UpdateData(UPDATE_CONTROL);
}

//ڍאݒ_CAOJ
void ColorPanel::OnColorDetail()
{
	ColorDetail dialog;
	dialog.DoModal(GetHWND());
}

APP_END_NAMESPACE
