#include "prefix.h"

#include "core.h"
#include "fpoint.h"
#include "particleinfolist.h"
#include "particleinfo.h"
#include "pi.h"
#include "image.h"
#include "channel.h"
#include "generalstatus.h"
#include "safeinteger.h"
#include "tool.h"
#include "colorinfo.h"
#include "cellsize.h"

#include <windows.h>
#include <math.h>
#include <assert.h>
#include <string.h>
#include <limits.h>
#include <memory>

/*************************************************

	q𑀂jS

**************************************************/

APP_BEGIN_NAMESPACE

//̗q̍ő吔
#define CELL_HOLDS_MAX	32

//q̑傫	f
#define PARTICLE_SIZE	8
//q̂P̃p^[̃oCg
#define PARTICLE_BYTES (PARTICLE_SIZE*PARTICLE_SIZE)
//q̕`p^[1hbgȉ̉𑜓x
#define PARTICLE_RESOLUTION 4

//qp^[̒S̔Z
#define PARTICLE_DEPTH	255

//߂ɂ闱q̔ZvMIX_LOWER_LIMIT
//MIX_RANGEƂ0-255Ɋtlۂ̉f̔ZɂȂ
//x̉
//ȉ͊SȓɂȂ
#define MIX_LOWER_LIMIT 180
//x̏-MIX_LOWER_LIMIT
//ȏ͊Sɓh
#define MIX_RANGE	256

//ʂ̒[ŗqˑRȂ悤ɂ邽߂̗]ȗ̈
#define DRAW_MARGIN	((PARTICLE_SIZE+CELL_SIZE-1)/CELL_SIZE)
//s̒[CɂȂ߂̗]ȗ̈
#define EMPTY_MARGIN	1

//q̏
struct PARTICLE
{
	unsigned short dx,dy;	//q̋̈ʒu
	float velocity_x,velocity_y;	//q̑x
	int age;	//o߃^[
	int type;	//e[u̔ԍ
	int generationLeft;	//̐㐔̎c萔

	PA_COLOR pickedColor;	//ɂ݃c[̂߂̐F
};

//qԗ͂vZ邽߂̃[NGǍ^
struct SUM
{
	FPOINT power;	//qɂĂ͂̍v
	FPOINT viscosity;	//qɂS̍v
	double efficacy;	//S̉ex̍v
};

#define SUM_ROW_COUNT 2

//̏
struct CELL	
{
	PARTICLE particles[CELL_HOLDS_MAX];	//̗q̏
	int count;	//̗q̐
};

class CoreMember
{
public:
	CoreMember()
		:tool(TOOL_PAINT)
	{
		imageSize.cx=imageSize.cy=0;
	}
	ParticleInfoList particleInfoList;	//q̑̔z
	Buffer<SUM> sum[SUM_ROW_COUNT];	//qԗ͂vZ邽߂̃[NGA


	SIZE imageSize;	//摜̑傫
	POINT imageOffset;	//s̍ォۂɕ\ʒu܂ł̃ItZbgB100%ł̃hbg
	SIZE matrixSize;	//s̑傫

	Buffer<CELL> matrix;		//q̏ԂLs
	Buffer<CELL> matrixBack;	//XVp̗s
	Buffer<DRAWWORK> work;	//[NGA obt@

	Buffer<unsigned char> particlePatterns;	//q̕`p^[

	int tool;	//qoʂ̎
};

Core::Core()
:m_m(new CoreMember)
{
}

Core::~Core() throw()
{

}

//摜̑傫
SIZE Core::GetSize() const
{
	return m_m->imageSize;
}

//摜̑傫ݒ
void Core::SetSize(SIZE size)
{
	m_m->imageSize=size;
}

//s
void ClearMatrix(CELL *cell,int count)
{
	FOR(int i=0;i<count;++i) {
		cell->count=0;
		++cell;
	}
}

//摜̑傫ɑ΂s̍s̃Z̐
inline const 
 int MatrixSize(int imageSize)
{
	return SafeAddition(imageSize,CELL_SIZE-1)/CELL_SIZE
		+EMPTY_MARGIN*2+DRAW_MARGIN;
}

//qԗ̓[NGA
inline
 void ClearSum(SUM *sum,int count)
{
	FOR(int i=0;i<count;++i) {
		sum->power.x=0;
		sum->power.y=0;
		sum->viscosity.x=0;
		sum->viscosity.y=0;
		sum->efficacy=0;
		++sum;
	}
}

//
void Core::Initialize()
{
	assert(0<m_m->imageSize.cx && 0<m_m->imageSize.cy);

	CreateParticlePattern();
	AllocWork();

	m_m->matrixSize.cx=MatrixSize(m_m->imageSize.cx);
	m_m->matrixSize.cy=MatrixSize(m_m->imageSize.cy);
	int cellCount=SafeMultiple(m_m->matrixSize.cx,m_m->matrixSize.cy);

	int matrixWidth=SafeMultiple(m_m->matrixSize.cx,CELL_SIZE);
	m_m->imageOffset.x=SafeSubtract(matrixWidth,m_m->imageSize.cx)/2;
	int matrixHeight=SafeMultiple(m_m->matrixSize.cy,CELL_SIZE);
	m_m->imageOffset.y=SafeSubtract(matrixHeight,m_m->imageSize.cy)/2;

	m_m->matrix.Alloc(cellCount);
	m_m->matrixBack.Alloc(cellCount);

	ClearMatrix(m_m->matrix.GetW(),cellCount);
	ClearMatrix(m_m->matrixBack.GetW(),cellCount);

	//Rsm
	int sumCount=SafeMultiple(m_m->matrixSize.cx,CELL_HOLDS_MAX);
	FOR(int i=0;i<SUM_ROW_COUNT;++i) {
		m_m->sum[i].Alloc(sumCount);
		ClearSum(m_m->sum[i].GetW(),sumCount);
	}
}

//c[ݒ
void Core::SetTool(int tool)
{
	Clear();
	m_m->tool=tool;
}

void Core::Progress(Image *image)	//Q[is
//߂l Q[̏
{
	//`
	Draw(image);
	
	//q̑x̌vZ
	CalcVelocity();
	//q̈ړ
	Move();
}

//̈ʒu̕\
#define CELLPOINT_MAX (SHRT_MAX+1)

//̈ʒuijunsigned shortɕϊ
inline const
 unsigned short CellPoint(double point)
{
	double small=point-floor(point);
	return CELLPOINT_MAX*small;
}

//̈ʒudoubleɕϊ
inline const
 double CellPoint(unsigned short point)
{
	assert(0<=point && point<CELLPOINT_MAX);
	return static_cast<double>(point)/CELLPOINT_MAX;
}

//q̑x̌vZ
void Core::CalcVelocity()
{
	CELL *cell=m_m->matrix.GetW();
	FOR(int y=0;y<m_m->matrixSize.cy;++y) {
		SUM *curSum=m_m->sum[y%SUM_ROW_COUNT].GetW();
		SUM *downSum=m_m->sum[(y+1)%SUM_ROW_COUNT].GetW();
		FOR(int x=0;x<m_m->matrixSize.cx;++x) {
			PARTICLE *particle=cell->particles;
			SUM *destSum=curSum;
			FOR(int h=0;h<cell->count;++h) {
				CalcVelocityParticle(x,y,h,cell,particle
					,destSum,curSum,downSum);
				++destSum;
				++particle;
			}
			curSum+=CELL_HOLDS_MAX;
			downSum+=CELL_HOLDS_MAX;
			++cell;
		}

#ifndef NDEBUG
		//gIs|Ă邩
		const SUM *row=m_m->sum[y % SUM_ROW_COUNT].Get();
		FOR(int x=0;x<m_m->matrixSize.cx*CELL_HOLDS_MAX;++x) {
			assert(row->power.x==0
				&& row->power.y==0
				&& row->viscosity.x==0
				&& row->viscosity.y==0
				&& row->efficacy==0);
			++row;
		}
#endif
	}
}

//̗̋q̑x̌vZ
void Core::CalcVelocityParticle(int x,int y,int number,CELL *cell
	,PARTICLE *particle,SUM *destSum,SUM *curSum,SUM *downSum)
//x,y ڂĂ̍W
//cell ڂĂ
//particle ڂĂ闱q
//destSum ڂĂ郏[NGA
//curSum ڂẴ[NGA
//downSum@Pĩ[NGA
{
#ifndef NDEBUG
	const CELL *dbg_cell=m_m->matrix.Get()+y*m_m->matrixSize.cx+x;
	assert(dbg_cell==cell);
	int dbg_number=(particle-dbg_cell->particles);
	assert(0<=dbg_number && dbg_number<CELL_HOLDS_MAX);
	assert(dbg_number==number);
	assert(cell->count>dbg_number);

	int dbg_sumNumber=destSum-(m_m->sum[y%SUM_ROW_COUNT].Get()+x*CELL_HOLDS_MAX);
	assert(0<=dbg_sumNumber && dbg_sumNumber<CELL_HOLDS_MAX);
	assert(dbg_number==dbg_sumNumber);

	dbg_sumNumber=curSum-(m_m->sum[y%SUM_ROW_COUNT].Get()+x*CELL_HOLDS_MAX);
	assert(dbg_sumNumber==0);

	dbg_sumNumber=downSum-(m_m->sum[(y+1)%SUM_ROW_COUNT].Get()+x*CELL_HOLDS_MAX);
	assert(dbg_sumNumber==0);

	assert(EMPTY_MARGIN<=x && x<m_m->matrixSize.cx-EMPTY_MARGIN
		&& EMPTY_MARGIN<=y && y<m_m->matrixSize.cy-EMPTY_MARGIN);
#endif

	//ڂĂ闱qƂȍ~̗qƂ̗͂vZ
	CalcPowerSurround(x,y,number,cell,particle,destSum,curSum,downSum);

	FPOINT velocity;
	velocity.x=particle->velocity_x;
	velocity.y=particle->velocity_y;

	const PARTICLE_INFO *particleInfo=ParticleInfo(particle);

	//]
	if(particleInfo->useSpin) {
		SpinParticle(particleInfo,particle,&velocity);
	}

	//S
	double efficacy=destSum->efficacy;
	if(efficacy>0) {
		FPOINT average;
		average.x=destSum->viscosity.x/efficacy;
		average.y=destSum->viscosity.y/efficacy;

		if(efficacy>1) efficacy=1;

		double rate=particleInfo->viscosity*efficacy;
		assert(0<=rate && rate<=1);
		velocity.x=velocity.x*(1-rate)+average.x*rate;
		velocity.y=velocity.y*(1-rate)+average.y*rate;
	}

	//qɂ͂ŉ
	velocity+=destSum->power;

	//Ŝ̏d
	FPOINT generalGravity;
	CalcGeneralGravity(&generalGravity
		,x+CellPoint(particle->dx),y+CellPoint(particle->dy));
	velocity+=generalGravity;

	//
	if(particleInfo->brake<1) {
		assert(particleInfo->brake>=0);
		velocity*=particleInfo->brake;
	}

	//x
	if(particleInfo->useLimitedSpeed && (velocity.x>0 || velocity.y>0)) {
		double curSpeed=hypot(velocity.x,velocity.y);
		if(curSpeed>particleInfo->limitedSpeed) {
			double rate=particleInfo->limitedSpeed/curSpeed;
			velocity*=rate;
		}
	}

	particle->velocity_x=static_cast<float>(velocity.x);
	particle->velocity_y=static_cast<float>(velocity.y);

	ClearSum(destSum,1);		//[NGÂǑ|
}

//q̑x̉]
void Core::SpinParticle(const PARTICLE_INFO *particleInfo,const PARTICLE *particle
	,FPOINT *velocity) const
{
	int age=particle->age;
	if(age==0) return;

	if((age % particleInfo->spinStep)!=0) return;

	SpinVelocity(velocity,particleInfo->spinAngle,particleInfo->spinAngleWiggle);
}

//x]
void Core::SpinVelocity(FPOINT *velocity,double angle,double angleWiggle) const
{
	if(angle==0 && angleWiggle==0) {
		return;
	}

	double wangle=angle+angleWiggle*rand()/RAND_MAX-angleWiggle/2;

	FPOINT oldVelocity(*velocity);
	double cosAngle=cos(wangle);
	double sinAngle=sin(wangle);
	velocity->x=oldVelocity.x*cosAngle + oldVelocity.y*sinAngle;
	velocity->y=-oldVelocity.x*sinAngle + oldVelocity.y*cosAngle;
}

//̋ɂ闱qƂ̗qԗ
void Core::CalcPowerSurround(int x,int y,int number
	,const CELL *cell,const PARTICLE *particle
	,SUM *destSum,SUM *curSum,SUM *downSum) const
//x,y	ڂĂ
//number ڂĂ闱q̋̔ԍ
//cell ڂĂ
//particle ڂĂ闱q
//destSum ڂĂ闱q̃[NGA
//curSum ڂẴ[NGA
//downSum@ĩ[NGA	
{
	//[̋ɂ͗qBȂ悤ɂĂł͔͈͌Ȃ
	assert(EMPTY_MARGIN<=x && x<m_m->matrixSize.cx-EMPTY_MARGIN
		&& EMPTY_MARGIN<=y && y<m_m->matrixSize.cy-EMPTY_MARGIN);

#ifndef NDEBUG
	const CELL *dbg_cell=m_m->matrix.Get()+y*m_m->matrixSize.cx+x;
	int dbg_number=(particle-dbg_cell->particles);
	assert(0<=dbg_number && dbg_number<CELL_HOLDS_MAX);
	assert(dbg_number==number);
#endif
	assert(curSum==m_m->sum[y%SUM_ROW_COUNT].Get()+x*CELL_HOLDS_MAX);
	assert(downSum==m_m->sum[(y+1)%SUM_ROW_COUNT].Get()+x*CELL_HOLDS_MAX);
	assert(destSum==curSum+number);

	//̗qƂȍ~̗qƂ̈͂vZ
	//
	CalcPowerCell(x,y,particle,destSum
		,x,y,cell,destSum+1
		,number+1);

	//E
	CalcPowerCell(x,y,particle,destSum
		,x+1,y,cell+1,curSum+CELL_HOLDS_MAX
		,0);
	
	//
	CalcPowerCell(x,y,particle,destSum
		,x-1,y+1,cell+m_m->matrixSize.cx-1,downSum-CELL_HOLDS_MAX
		,0);
	
	//
	CalcPowerCell(x,y,particle,destSum
		,x,y+1,cell+m_m->matrixSize.cx,downSum
		,0);

	//E
	CalcPowerCell(x,y,particle,destSum
		,x+1,y+1,cell+m_m->matrixSize.cx+1,downSum+CELL_HOLDS_MAX
		,0);
}

//qԗ͂[NGAɉ
void Core::CalcPowerCell(int x,int y,const PARTICLE *particle,SUM *destSum
	,int near_x,int near_y,const CELL *nearCell,SUM *nearSum
	, int offset) const
//̋ɑ΂闱qԓ͂̌vZ
//x,y	ڂĂ̍W
//particle		ڂĂ闱q
//destSum	ڂĂ闱q̃[NGA
//near_x,near_y	߂ɂ̍W
//nearCell ߂ɂ
//nearSum ߂ɂ闱q̃[NGA
//offset	ڂĂƓj̏ꍇ̐擪̃ItZbg
{
	const int nearCount=nearCell->count;
	assert(0<=nearCount && nearCount<=CELL_HOLDS_MAX);
	assert(0<=offset);
	if(offset>=nearCount) return;	//qȂȂA܂傤

#ifndef NDEBUG
	const CELL *dbg_cell=m_m->matrix.Get()+y*m_m->matrixSize.cx+x;
	int dbg_number=(particle-dbg_cell->particles);
	assert(0<=dbg_number && dbg_number<CELL_HOLDS_MAX);
	assert(offset==0 || dbg_number==offset-1);
	assert(dbg_cell->count>dbg_number);

	int dbg_sumNumber=destSum-(m_m->sum[y%SUM_ROW_COUNT].Get()+x*CELL_HOLDS_MAX);
	assert(0<=dbg_sumNumber && dbg_sumNumber<CELL_HOLDS_MAX);
	assert(dbg_number==dbg_sumNumber);

	dbg_cell=m_m->matrix.Get()+near_y*m_m->matrixSize.cx+near_x;
	assert(dbg_cell==nearCell);
	dbg_sumNumber=nearSum-(m_m->sum[near_y%SUM_ROW_COUNT].Get()+near_x*CELL_HOLDS_MAX);
	assert(offset==CELL_HOLDS_MAX ||(0<=dbg_sumNumber && dbg_sumNumber<CELL_HOLDS_MAX));
	assert(dbg_cell->count>dbg_sumNumber);

	assert(EMPTY_MARGIN<=x && x<m_m->matrixSize.cx-EMPTY_MARGIN
		&& EMPTY_MARGIN<=y && y<m_m->matrixSize.cy-EMPTY_MARGIN);
	assert(0<=near_x && near_x<m_m->matrixSize.cx
		&& 0<=y && y<m_m->matrixSize.cy);
#endif

	FPOINT powerSum(destSum->power);
	FPOINT viscositySum(destSum->viscosity);
	double efficacySum=destSum->efficacy;

	FPOINT destVelocity;
	destVelocity.x=particle->velocity_x;
	destVelocity.y=particle->velocity_y;


	//炩ߋ߂ɂ闱q̋̍WA
	//ڂĂ闱q̍WĂ
	FPOINT destPointD;
	destPointD.x=(near_x-x)-CellPoint(particle->dx);
	destPointD.y=(near_y-y)-CellPoint(particle->dy);

	const PARTICLE_INFO *destType=ParticleInfo(particle);
	double destRepulsion=destType->repulsion;
	double repulsionArea=destType->repulsionArea;
	double destGravity=destType->gravity;
	double gravityArea=destType->gravityArea;
	double destViscosity=destType->viscosity;
	double viscosityArea=destType->viscosityArea;

	const PARTICLE *nearParticle=&(nearCell->particles[offset]);

	FOR(int i=offset;i<nearCount;++i) {
		double xdistance=CellPoint(nearParticle->dx)+destPointD.x;
		double ydistance=CellPoint(nearParticle->dy)+destPointD.y;
		//q̋^2
		double distance2=xdistance*xdistance+ydistance*ydistance;

		if(distance2<=1) {
			FPOINT powerVector;
			if(distance2>0) {
				const PARTICLE_INFO *nearType=ParticleInfo(nearParticle);
				double distance=sqrt(distance2);
				double power=0;

				//͈͓͂̔
				double gravityR=1-distance*gravityArea;
				if(gravityR>0) {
					assert(gravityR<=1);
					power+=(destGravity*nearType->gravity) * gravityR;
				}

				//
				double repulsionR2=1-distance2*repulsionArea;
				if(repulsionR2>0) {
					assert(repulsionR2<=1);
					power-=(destRepulsion*nearType->repulsion) * repulsionR2;
				}

				//͂wxɕ
				powerVector.x=power*xdistance/distance;
				powerVector.y=power*ydistance/distance;

				//Ŝ߂ɑx̍vB
				double viscosityR=1-distance*viscosityArea;
				if(viscosityR>0) {
					double destEfficacy=nearType->viscosity*viscosityR;
					viscositySum.x+=nearParticle->velocity_x*destEfficacy;
					viscositySum.y+=nearParticle->velocity_y*destEfficacy;
					efficacySum+=destEfficacy;

					double nearEfficacy=destViscosity*viscosityR;
					nearSum->viscosity.x+=destVelocity.x*nearEfficacy;
					nearSum->viscosity.y+=destVelocity.y*nearEfficacy;
					nearSum->efficacy+=nearEfficacy;
				}
			} else {
				//ꏊɂ	ĂƂ
				if(destGravity>0 || destRepulsion>0) {
					powerVector.x=rand()*(0.01/RAND_MAX);
					powerVector.y=rand()*(0.01/RAND_MAX);
				} else {
					powerVector.x=powerVector.y=0;
				}
			}
			//ڂĂ闱q̈͂ɍv
			powerSum+=powerVector;

			//̗qɈ͂𑫂Ă	vZ鑊Ƃ͋tȂ̂Ń}CiX
			nearSum->power-=powerVector;
		}

		++nearParticle;
		++nearSum;
	}

	destSum->power=powerSum;
	destSum->viscosity=viscositySum;
	destSum->efficacy=efficacySum;
}

//Ŝɂd͂vZ
void Core::CalcGeneralGravity(FPOINT *gravity,double x,double y) const
{
	const GENERAL_STATUS *generalStatus=GetGeneralStatus();

	int type=generalStatus->gravityType;
	if(type==GRAVITY_PARALLEL) {
		//s
		*gravity=generalStatus->cellGravity;
	} else {
		assert(type==GRAVITY_RADIAL);

		//~
		x-=m_m->matrixSize.cx/2;
		y-=m_m->matrixSize.cy/2;

		double length=hypot(x,y);
		if(length>0) {
			FPOINT generalGravity(generalStatus->cellGravity);
			gravity->x=(-y*generalGravity.x
				- x*generalGravity.y)/length;
			gravity->y=(x*generalGravity.x
				- y*generalGravity.y)/length;
		} else {
			gravity->x=gravity->y=0;
		}
	}
}

//q̈ړ
void Core::Move()
{
	CELL *cell=m_m->matrix.GetW();

	//SĂ̗qړ
	FOR(int y=0;y<m_m->matrixSize.cy;++y) {
		FOR(int x=0;x<m_m->matrixSize.cx;++x) {
			const int count=cell->count;
			assert(0<=count && count<=CELL_HOLDS_MAX);
			if(count>0) {
				cell->count=0;	//Âz͑|
				PARTICLE *particle=cell->particles;
				FOR(int h=0;h<count;++h) {
					//q̈ړ
					MoveAndSplitParticle(x,y,particle);
					++particle;
				}
			}
			++cell;
		}
	}

#ifndef NDEBUG
	const CELL *debug_cell=m_m->matrix.Get();
	FOR(int i=0;i<m_m->matrixSize.cx*m_m->matrixSize.cy;++i) {
		assert(debug_cell->count==0);
		++debug_cell;
	}
#endif

	//O̔zƓւ
	m_m->matrix.Swap(m_m->matrixBack);
}

//q͈͊Oɏo
bool Core::IsParticleDropped(int x,int y) const
//x y W̋̈ʒu
{
	return x<EMPTY_MARGIN
		|| (m_m->matrixSize.cx-EMPTY_MARGIN)<=x
		|| y<EMPTY_MARGIN
		|| (m_m->matrixSize.cy-EMPTY_MARGIN)<=y;
}

//qړƕėsɓ
void Core::MoveAndSplitParticle(int x,int y,const PARTICLE *particle)
//x,y 
//particle ړ闱q
{
	if(IsParticleDead(particle)) {
		DecreaseReferenceParticleInfo(particle->type);
		return;
	}

	const PARTICLE_INFO *particleInfo=ParticleInfo(particle);

	int age=particle->age;
	int generationLeft=particle->generationLeft;
	if(!particleInfo->useSplit || age==0 || generationLeft<=0
		|| particleInfo->splitRate<(static_cast<double>(rand())/RAND_MAX)) {
		MoveParticle(x,y,particle,0,0,generationLeft);
		DecreaseReferenceParticleInfo(particle->type);
		return;
	}
	
	int mode=particleInfo->splitMode;
	double angleWiggle=particleInfo->splitAngleWiggle;

	int step1=particleInfo->splitStep1;
	bool matchStep1=(age % step1)==0;
	if(matchStep1) {
		MoveParticle(x,y,particle,particleInfo->splitAngle1,angleWiggle,generationLeft-1);
	} 

	bool isForked=(matchStep1 && mode==SPLIT_FORKED);
	if(!isForked) {
		MoveParticle(x,y,particle,0,0,generationLeft);		
	}

	int step2=particleInfo->splitStep2;
	int lag=particleInfo->splitLag;
	bool matchStep2=isForked
		|| (mode==SPLIT_DOUBLE  &&  ((age-lag) % step2)==0);
	if(matchStep2) {
		MoveParticle(x,y,particle,particleInfo->splitAngle2,angleWiggle,generationLeft-1);
	}

	DecreaseReferenceParticleInfo(particle->type);
}

//x]Ă痱qړ
void Core::MoveParticle(int x,int y,const PARTICLE *particle
		,double angle,double angleWiggle,int generationLeft)
{
	FPOINT velocity;
	velocity.x=particle->velocity_x;
	velocity.y=particle->velocity_y;
	SpinVelocity(&velocity,angle,angleWiggle);

	FPOINT point;
	point.x=x+CellPoint(particle->dx);
	point.y=y+CellPoint(particle->dy);

		//xňړ
	double generalLively=GetGeneralStatus()->lively;
	point.x+=velocity.x*generalLively;
	point.y+=velocity.y*generalLively;

	PutToBackMatrix(particle,point,velocity,generationLeft);
}

//sɗqu
void Core::PutToBackMatrix(const PARTICLE *particle,const FPOINT &point
	,const FPOINT &velocity,int generationLeft)
{
	int ix=int(point.x);
	int iy=int(point.y);

	if(IsParticleDropped(ix,iy)) {
		return;
	}

	CELL *cell=m_m->matrixBack.GetW()+iy*m_m->matrixSize.cx+ix;
	int count=cell->count;
	assert(0<=count && count<=CELL_HOLDS_MAX);
	if(count>=CELL_HOLDS_MAX) {
		return;
	}

	//ɋ󂫂Εێ𑝂₵ē
	++cell->count;
	PARTICLE *newParticle=&(cell->particles[count]);
	newParticle->dx=CellPoint(point.x);
	newParticle->dy=CellPoint(point.y);
	newParticle->velocity_x=static_cast<float>(velocity.x);
	newParticle->velocity_y=static_cast<float>(velocity.y);
	newParticle->type=particle->type;
	newParticle->age=particle->age+1;
	newParticle->generationLeft=generationLeft;
	newParticle->pickedColor=particle->pickedColor;

	m_m->particleInfoList.IncreaseReferenceCount(newParticle->type);
}

//q̎ނ̏
const PARTICLE_INFO *Core::ParticleInfo(const PARTICLE *particle) const
{
	return m_m->particleInfoList.Get(particle->type);
}

//s
bool Core::IsParticleDead(const PARTICLE *particle) const
{
	const PARTICLE_INFO *particleInfo=ParticleInfo(particle);
	return particle->age>=particleInfo->life;
}

//qu
void Core::Put(const PUT_PARAM &putParam)
{
	FPOINT velocity(putParam.velocity);
	FPOINT imagePoint(putParam.point);

	//ʒuɑėƂȂ߂ɗ
	imagePoint.x+=static_cast<double>
		(rand())/RAND_MAX;
	imagePoint.y+=static_cast<double>
		(rand())/RAND_MAX;
	
	FPOINT putPoint(
		ImageToMatrix(imagePoint.x,imagePoint.y));

	int ix=static_cast<int>(putPoint.x);
	int iy=static_cast<int>(putPoint.y);

	//Oɔ˂Ȃ牽Ȃ
	if(IsParticleDropped(ix,iy)) {
		return;
	}

	//ɐVqǉ
	CELL *cell=m_m->matrix.GetW()+iy*m_m->matrixSize.cx+ix;
	int count=cell->count;
	assert(0<=count && count<=CELL_HOLDS_MAX);
	if(count>=CELL_HOLDS_MAX) {
		//̍őێɒBĂꍇ͉Ȃ
		return;
	}
	PARTICLE *particle=&(cell->particles[count]);
	particle->type=putParam.particleInfo;
	particle->dx=CellPoint(putPoint.x);
	particle->dy=CellPoint(putPoint.y);
	particle->velocity_x=static_cast<float>(velocity.x);
	particle->velocity_y=static_cast<float>(velocity.y);
	particle->age=0;
	particle->generationLeft=putParam.splitGenerationsMax;

	if(m_m->tool==TOOL_BLUR) {
		particle->pickedColor=PickPixelColor(putParam.image,imagePoint.x,imagePoint.y);
	}

	m_m->particleInfoList.IncreaseReferenceCount(putParam.particleInfo);
	++cell->count;
}

//`̕
inline int Width(const RECT *rect)
{
	return rect->right-rect->left;
}

//摜̍Ws̍Wɕϊ
FPOINT Core::ImageToMatrix(double x,double y) const
{
	FPOINT matrixPoint;
	matrixPoint.x=(m_m->imageOffset.x+x)/CELL_SIZE;
	matrixPoint.y=(m_m->imageOffset.y+y)/CELL_SIZE;
	return matrixPoint;
}

//摜̍W琮̍s̍Wɕϊ
POINT Core::ImageToMatrixInt(double x,double y) const
{
	FPOINT matrixPoint(ImageToMatrix(x,y));
	
	POINT matrixPointInt;
	matrixPointInt.x=static_cast<int>(matrixPoint.x);
	matrixPointInt.y=static_cast<int>(matrixPoint.y);
	return matrixPointInt;
}

inline const
 int MatrixToImageInt(int offset,int pos)
{
	return pos*CELL_SIZE-offset;
}

//s̍W\Wɕϊ
void Core::MatrixToImage(FPOINT *imagePoint,const FPOINT &matrixPoint) const
{
	imagePoint->x=matrixPoint.x*CELL_SIZE-m_m->imageOffset.x;
	imagePoint->y=matrixPoint.y*CELL_SIZE-m_m->imageOffset.y;
}

//[NGA
struct DRAWWORK
{
	unsigned short depth;
	PA_COLOR color;
};

//[NGA̍s
#define WORK_ROW_COUNT_BIT 6
#define WORK_ROW_COUNT (1<<WORK_ROW_COUNT_BIT)
#if WORK_ROW_COUNT<CELL_SIZE+PARTICLE_SIZE
# error too little WORK_ROW_COUNT_BIT
#endif

//\
void Core::Draw(Image *image)
{
	int height=image->Height();
	assert(image->Width()==m_m->imageSize.cx && height==m_m->imageSize.cy);
	ClearWork(0,WORK_ROW_COUNT);

	CELL *cell=m_m->matrix.GetW();

	//p^[̈ԏɂꍇ̏cW
	int mostTop=MatrixToImageInt(m_m->imageOffset.y,0)
		-PARTICLE_SIZE/2;
	if(mostTop<0) mostTop=0;

	FOR(int y=0;y<m_m->matrixSize.cy;++y) {
		FOR(int x=0;x<m_m->matrixSize.cx;++x) {
			PARTICLE *particle=cell->particles;
			FOR(int h=0;h<cell->count;++h) {
				DrawParticle(image,x,y,particle);
				++particle;
			}
			++cell;
		}
		//s̎̍sŗq`ԏ̉摜̍s
		int nextMostTop=MatrixToImageInt(m_m->imageOffset.y,y+1)
			-PARTICLE_SIZE/2;
		if(nextMostTop>height) {
			nextMostTop=height;
		}
		//s̎̍sł͕`\̂Ȃm蕔摜ƍă[NGA
		int mixRowCount=nextMostTop-mostTop;
		if(mixRowCount>0) {
			MixImage(image,mostTop,mixRowCount);
			ClearWork(mostTop,mixRowCount);
			mostTop=nextMostTop;
		}
	}

	int mixRowCount=height-mostTop;
	if(mixRowCount>0) {
		MixImage(image,mostTop,mixRowCount);
	}
}

//vύX@߂ɗq̏Ȃ̂قǔZȂ悤ɂ

//q`
void Core::DrawParticle(const Image *image,int x,int y,PARTICLE *particle)
{
	//sł̍W
	FPOINT matrixPoint;
	matrixPoint.x=x+CellPoint(particle->dx);
	matrixPoint.y=y+CellPoint(particle->dy);

	//摜ł̍W
	FPOINT imagePoint;
	MatrixToImage(&imagePoint,matrixPoint);

	//͉f̈ʒuA_ȉ̓p^[̂B
	int x_int=static_cast<int>(floor(imagePoint.x));
	int x_float=static_cast<int>((imagePoint.x-x_int)*PARTICLE_RESOLUTION);
	assert(x_int<=imagePoint.x);
	assert(0<=x_float && x_float < PARTICLE_RESOLUTION);

	int y_int=static_cast<int>(floor(imagePoint.y));
	int y_float=static_cast<int>((imagePoint.y-y_int)*PARTICLE_RESOLUTION);
	assert(y_int<=imagePoint.y);
	assert(0<=y_float && y_float < PARTICLE_RESOLUTION);

	PA_COLOR color;
	color=ParticleColor(particle);

	if(m_m->tool==TOOL_BLUR) {
		particle->pickedColor=PickPixelColor(image,x_int,y_int);
	}

	//Phbgȉ̂̃p^[̃AhX
	const unsigned char *pattern=m_m->particlePatterns.Get()
		+PARTICLE_BYTES*(y_float*PARTICLE_RESOLUTION+x_float);

	//NbsOĕ``
	int left=x_int-PARTICLE_SIZE/2;
	if(left<0) {
		pattern+=-left;
		left=0;
	}

	int right=x_int+PARTICLE_SIZE-PARTICLE_SIZE/2;
	if(right>m_m->imageSize.cx) {
		right=m_m->imageSize.cx;
	}

	int top=y_int-PARTICLE_SIZE/2;
	if(top<0) {
		pattern+=(-top)*PARTICLE_SIZE;
		top=0;
	}

	int bottom=y_int+PARTICLE_SIZE-PARTICLE_SIZE/2;
	if(bottom>m_m->imageSize.cy) {
		bottom=m_m->imageSize.cy;
	}

	//p^[̃|C^̍sɐi߂ItZbg
	const int drawWidth=right-left;
	const int drawHeight=bottom-top;
	if(drawWidth<=0 || drawHeight<=0) return;
	assert(drawWidth<=PARTICLE_SIZE && drawHeight<=PARTICLE_SIZE);

	unsigned long particleOpacity=color.channel[CHANNEL_OPACITY];

	int patternNextRow=PARTICLE_SIZE-drawWidth;
	FOR(int y=0;y<drawHeight;++y) {
		//[NGÃAhX
		DRAWWORK *work=GetWorkRowPtr(top+y)+left;

		FOR(int x=0;x<drawWidth;++x) {
			unsigned long patternDepth=*pattern;
			++pattern;

			unsigned long workDepth=work->depth;
			unsigned long workOpacity=work->color.channel[CHANNEL_OPACITY];

			unsigned long workAlpha=workDepth*workOpacity;
			unsigned long particleAlpha=patternDepth*particleOpacity;

			if(particleAlpha>0) {
				if(workAlpha>0) {
					unsigned long bothAlpha=workAlpha+particleAlpha;
					FOR(int c=0;c<COLOR_BYTES;++c) {
						unsigned long workColor=work->color.channel[c];
						unsigned long particleColor=color.channel[c];
						work->color.channel[c]=(workColor*workAlpha
							+particleColor*particleAlpha)
							 /bothAlpha;
					}
				} else {
					work->color=color;
				}
			}

			if(patternDepth>0) {
				if(workDepth>0) {
					workOpacity=(workAlpha+particleAlpha)
						/(workDepth+patternDepth);
				} else {
					workOpacity=particleOpacity;
				}
				work->color.channel[CHANNEL_OPACITY]=workOpacity;
			}

			workDepth+=patternDepth;
			if(workDepth>USHRT_MAX) {
				workDepth=USHRT_MAX;
			}
			work->depth=workDepth;

			++work;
		}

		pattern+=patternNextRow;
	}
}

//q̐F
PA_COLOR Core::ParticleColor(const PARTICLE *particle) const
{
	const PARTICLE_INFO *particleInfo=ParticleInfo(particle);
	PA_COLOR color;
	switch(particleInfo->colorInfo.coloring) {
	case COLORING_SINGLE:
		color=ParticleColor_Single(particleInfo);
		break;

	case COLORING_GRADATION:
		color=ParticleColor_Gradation(particle,particleInfo);
		break;
		
	case COLORING_CYCLE:
		color=ParticleColor_Cycle(particle,particleInfo);
		break;

	case COLORING_RANDOM:
		color=ParticleColor_Random(particleInfo);
		break;

	default:
		assert(false);
	}

	if(m_m->tool==TOOL_BLUR) {
		color=MixPickedColor(particle,color);
	}

	return color;
}

//ɂ݂̂߂ɏEF
PA_COLOR Core::MixPickedColor(const PARTICLE *particle,PA_COLOR color) const
{
	PA_COLOR picked(particle->pickedColor);

	int opacity=color.channel[CHANNEL_OPACITY];
	if(opacity>0) ++opacity;
	opacity=opacity*picked.channel[CHANNEL_OPACITY]
		/(UCHAR_MAX+1);
	picked.channel[CHANNEL_OPACITY]=opacity;
	return picked;
}

//PFʂ
PA_COLOR Core::ParticleColor_Single(const PARTICLE_INFO *particleInfo) const
{
	PA_COLOR color;
	if(m_m->tool!=TOOL_PAINT || particleInfo->colorInfo.isForeSelected) {
		color=particleInfo->colorInfo.color[FORECOLOR];
	} else {
		color=particleInfo->colorInfo.color[BACKCOLOR];
	}
	return color;
}

//OiFwiFւƕς
PA_COLOR Core::ParticleColor_Gradation(const PARTICLE *particle
	,const PARTICLE_INFO *particleInfo) const
{
	int life=particleInfo->life;
	PA_COLOR color(particleInfo->colorInfo.color[FORECOLOR]);
	if(life<=0) {
		return color;
	}

	int age=particle->age;
	PA_COLOR alterColor(particleInfo->colorInfo.color[BACKCOLOR]);
	if(age>=life) {
		return alterColor;
	}

	FOR(int c=0;c<PA_COLOR_BYTES;++c) {
		int mix=(color.channel[c]*(life-age)
			+alterColor.channel[c]*age)/life;
		color.channel[c]=static_cast<unsigned char>(mix);
	}
	return color;
}

//Ȗ͗lŕςF
PA_COLOR Core::ParticleColor_Cycle(const PARTICLE *particle
	,const PARTICLE_INFO *particleInfo) const
{
	int cycleLength=particleInfo->colorInfo.cycleLength;
	int age=particle->age;
	int rate=age % cycleLength;
	if(rate>cycleLength/2) {
		rate=cycleLength-rate;
	}
	assert(0<=rate && rate<=cycleLength/2);
	PA_COLOR color(particleInfo->colorInfo.color[FORECOLOR]);
	PA_COLOR alterColor(particleInfo->colorInfo.color[BACKCOLOR]);

	FOR(int c=0;c<PA_COLOR_BYTES;++c) {
		int mix=(color.channel[c]*(cycleLength/2-rate)
			+alterColor.channel[c]*rate)/(cycleLength/2);
		color.channel[c]=static_cast<unsigned char>(mix);
	}
	return color;
}

//PFʂ
PA_COLOR Core::ParticleColor_Random(const PARTICLE_INFO *particleInfo) const
{
	return particleInfo->colorInfo.color[FORECOLOR];
}

//f̐F𓾂
PA_COLOR Core::PickPixelColor(const Image *image,int x,int y) const
{
	if(x<0) x=0;
	if(m_m->imageSize.cx<=x) x=m_m->imageSize.cx-1;
	if(y<0) y=0;
	if(m_m->imageSize.cy<=y) y=m_m->imageSize.cy-1;

	const PA_COLOR *pixel=(const PA_COLOR *)(image->GetPixel(x,y));
	return *pixel;	
}

typedef void (*MixImageProc)(unsigned char *imageptr,PA_COLOR workColor,unsigned long workAlpha);

static void MixImage_Paint(unsigned char *,PA_COLOR,unsigned long);
static void MixImage_Erase(unsigned char *,PA_COLOR,unsigned long);

//摜Ƀ[NGA̓e
void Core::MixImage(Image *image,int mostTop,int rowCount)
{
	assert(0<=mostTop);
	assert(1<=rowCount && rowCount<=WORK_ROW_COUNT);

	unsigned char *imageptr=image->GetRowW(mostTop);
	assert(image->PixelBytes()==PIXEL_BYTES);
	assert(image->PadBytes()==0);
	const int width=image->Width();
	assert(width==m_m->imageSize.cx);
	assert(mostTop+rowCount<=m_m->imageSize.cy);
	assert(image->Height()==m_m->imageSize.cy);

	MixImageProc toolProc;
	switch(m_m->tool) {
	case TOOL_PAINT:
	case TOOL_BLUR:
		toolProc=MixImage_Paint;
		break;

	case TOOL_ERASE:
		toolProc=MixImage_Erase;
		break;

	default:
		assert(false);
	}

	FOR(int y=0;y<rowCount;++y) {
		const DRAWWORK *work=GetWorkRowPtr(mostTop+y);
		FOR(int x=0;x<width;++x) {
			unsigned long workDepth=work->depth;
			if(workDepth>MIX_LOWER_LIMIT) {
				workDepth-=MIX_LOWER_LIMIT;
				if(workDepth>MIX_RANGE) {
					workDepth=MIX_RANGE;
				}
			} else {
				workDepth=0;
			}

			unsigned long workOpacity=work->color.channel[CHANNEL_OPACITY];
			if(workOpacity>0) ++workOpacity;
			unsigned long workAlpha=workOpacity*workDepth;

			if(workAlpha>0) {
				(*toolProc)(imageptr,work->color,workAlpha);
			}
			++work;
			imageptr+=PIXEL_BYTES;
		}
	}
}

//MixImage̓hƂɂ݃c[̏
static void MixImage_Paint(unsigned char *imageptr,PA_COLOR workColor,unsigned long workAlpha)
{
	unsigned long imageOpacity=imageptr[CHANNEL_OPACITY];
	unsigned long imageAlpha=MIX_RANGE*imageOpacity;
	unsigned long bothAlpha=imageAlpha+workAlpha;
	FOR(int c=0;c<COLOR_BYTES;++c) {
		unsigned long imageDepth=imageptr[c];
		unsigned long workDepth=workColor.channel[c];
		imageDepth=(imageDepth*imageAlpha
			+workDepth*workAlpha)
			 /bothAlpha;
		imageptr[c]=static_cast<unsigned char>(imageDepth);
	}
	imageOpacity=(imageOpacity*(MIX_RANGE*(UCHAR_MAX+1)-workAlpha)
		+UCHAR_MAX*workAlpha)
			/(MIX_RANGE*(UCHAR_MAX+1));
	assert(0<=imageOpacity && imageOpacity<=UCHAR_MAX);
	imageptr[CHANNEL_OPACITY]=imageOpacity;
}

//MixImagȅc[̏
static void MixImage_Erase(unsigned char *imageptr,PA_COLOR,unsigned long workAlpha)
{
	unsigned long imageOpacity=imageptr[CHANNEL_OPACITY];

	imageOpacity=(imageOpacity*(MIX_RANGE*(UCHAR_MAX+1)-workAlpha))
		/(MIX_RANGE*(UCHAR_MAX+1));
	assert(0<=imageOpacity && imageOpacity<=UCHAR_MAX);
	imageptr[CHANNEL_OPACITY]=imageOpacity;
}


//[NGAm
void Core::AllocWork()
{
	int count=SafeMultiple(m_m->imageSize.cx,WORK_ROW_COUNT);
	m_m->work.Increase(count);
}

//[NGA
void Core::ClearWork(int mostTop,int rowCount)
{
	int workTop = mostTop % WORK_ROW_COUNT;
	if(workTop+rowCount>WORK_ROW_COUNT) {
		int tailCount=WORK_ROW_COUNT-workTop;
		int headCount=rowCount-tailCount;
		memset(m_m->work.GetW(),0
			,headCount*m_m->imageSize.cx*sizeof(DRAWWORK));
		memset(GetWorkRowPtr(workTop),0
			,tailCount*m_m->imageSize.cx*sizeof(DRAWWORK));

	} else {
		memset(GetWorkRowPtr(workTop),0
			,rowCount*m_m->imageSize.cx*sizeof(DRAWWORK));
	}
}

//[NGA̍s̐擪AhX
DRAWWORK *Core::GetWorkRowPtr(int y)
{
	return m_m->work.GetW()+m_m->imageSize.cx*(y % WORK_ROW_COUNT);
}

//g債qp^[̑傫
#define PARTICLE_BIG_SIZE ((PARTICLE_SIZE+1)*PARTICLE_RESOLUTION)
#define PARTICLE_BIG_BYTES (PARTICLE_BIG_SIZE*PARTICLE_BIG_SIZE)

//q̕`p^[쐬
void Core::CreateParticlePattern()
{
	Buffer <unsigned char> bigPattern;
	bigPattern.Alloc(PARTICLE_BIG_BYTES);
	CreateBigPattern(bigPattern.GetW());

	const int bufferSize=PARTICLE_BYTES
		*PARTICLE_RESOLUTION*PARTICLE_RESOLUTION;
	m_m->particlePatterns.Alloc(bufferSize);

	CreateSmallPatterns(m_m->particlePatterns.GetW(),bigPattern.Get());
}

//g債qp^[̉~̔a
#define PARTICLE_BIG_R (((PARTICLE_SIZE-1)*PARTICLE_RESOLUTION+1)/2.0)

//Phbgȉ̂邽߂ɗq̑傫p^[
void Core::CreateBigPattern(unsigned char *buffer)
//buffer obt@
{
	FOR(int y=0;y<PARTICLE_BIG_SIZE;++y) {
		FOR(int x=0;x<PARTICLE_BIG_SIZE;++x) {
			int rx=x-PARTICLE_BIG_R;
			int ry=y-PARTICLE_BIG_R;

			double r;
			r=hypot(rx,ry)/PARTICLE_BIG_R;

			double d=0;
			if(r<1) {
				d=(1-cos(PI*(1-r)))/2;
				d=d*PARTICLE_DEPTH+1;
				if(d>PARTICLE_DEPTH) {
					d=PARTICLE_DEPTH;
				}
			}
			*buffer=static_cast<char>(d);
			++buffer;
		}
	}
}

//1hbgȉ̂邽߂ɑ傫p^[炵
//p^[
void Core::CreateSmallPatterns(unsigned char *smallPatterns,const unsigned char *bigPattern)
//smallPatterns	p^[̃obt@
//bigPattern	傫p^[̃obt@
{
	FOR(int y=0;y<PARTICLE_RESOLUTION;++y) {
		FOR(int x=0;x<PARTICLE_RESOLUTION;++x) {
			ReducePattern(smallPatterns
				,bigPattern+(PARTICLE_RESOLUTION-x)
					+(PARTICLE_RESOLUTION-y)*PARTICLE_BIG_SIZE);
			smallPatterns+=PARTICLE_BYTES;
		}
	}
}

//p^[k
void Core::ReducePattern(unsigned char *smallPattern,const unsigned char *bigPattern)
//smallPattern	kp^[̃obt@
//bigPattern	傫p^[̃obt@
{
	FOR(int y=0;y<PARTICLE_SIZE;++y) {
		FOR(int x=0;x<PARTICLE_SIZE;++x) {
			int depthSum=0;
			FOR(int ry=0;ry<PARTICLE_RESOLUTION;++ry) {
				FOR(int rx=0;rx<PARTICLE_RESOLUTION;++rx) {
					depthSum+=bigPattern[(x*PARTICLE_RESOLUTION+rx)
						+(y*PARTICLE_RESOLUTION+ry)*PARTICLE_BIG_SIZE];
				}
			}

			*smallPattern=depthSum
				/(PARTICLE_RESOLUTION*PARTICLE_RESOLUTION);
			++smallPattern;
		}
	}
}

//q̑o^
int Core::AddParticleInfo(const PARTICLE_INFO &particleInfo)
{
	return m_m->particleInfoList.AddType(particleInfo);
}

//q̑̎QƐ炷
void Core::DecreaseReferenceParticleInfo(int particleInfo)
{
	m_m->particleInfoList.DecreaseReferenceCount(particleInfo);
}

//q݂邩
bool Core::HasParticle() const
{
	return !m_m->particleInfoList.IsEmpty();
}

//SĂ̗q폜
void Core::Clear()
{
	CELL *cell=m_m->matrix.GetW();

	FOR(int y=0;y<m_m->matrixSize.cy;++y) {
		FOR(int x=0;x<m_m->matrixSize.cx;++x) {
			const int count=cell->count;
			assert(0<=count && count<=CELL_HOLDS_MAX);
			if(count>0) {
				PARTICLE *particle=cell->particles;
				FOR(int h=0;h<count;++h) {
					DecreaseReferenceParticleInfo(particle->type);
					++particle;
				}
			}
			cell->count=0;
			++cell;
		}
	}
}

APP_END_NAMESPACE
