#ifndef APP_CORE_H
#define APP_CORE_H

#ifdef _MSC_VER
#pragma once
#endif

#include "site.h"
#include "fpoint.h"
#include "channel.h"

#include <windows.h>

/*************************************************

	q𑀂jS

**************************************************/

APP_BEGIN_NAMESPACE

class Image;

//˂̎d
struct PUT_PARAM
{
	FPOINT velocity;	//
	FPOINT point;	//W
	const Image *image;	//摜
	int particleInfo;	//q̔ԍ
	int splitGenerationsMax;	//̍ő吢㐔
};

struct CELL;
struct SUM;
struct PARTICLE;
struct PARTICLE_INFO;
struct DRAWWORK;
union PA_COLOR;

class ParticleInfoList;

class Core
{
public:
	Core();
	~Core() throw();

	void SetSize(SIZE size);
	SIZE GetSize() const;
	void Initialize();
	void SetTool(int tool);

	void Progress(Image *image);
	void Put(const PUT_PARAM &putParam);

	int AddParticleInfo(const PARTICLE_INFO &particleInfo);
	void DecreaseReferenceParticleInfo(int particleInfo);

	bool HasParticle() const;
	void Clear();

protected:
	void CalcVelocity();
	void CalcVelocityParticle(int x,int y,int number,CELL *cell
		,PARTICLE *particle,SUM *destSum,SUM *curSum,SUM *downSum);
	void CalcPowerSurround(int x,int y,int number
		,const CELL *cell,const PARTICLE *particle
		,SUM *destSum,SUM *curSum,SUM *downSum) const;
	void CalcPowerCell(int x,int y,const PARTICLE *particle,SUM *destSum
		,int near_x,int near_y,const CELL *nearCell,SUM *nearSum
		,int offset) const;

	void SpinParticle(const PARTICLE_INFO *particleInfo,const PARTICLE *particle
		,FPOINT *velocity) const;
	void SpinVelocity(FPOINT *velocity,double angle,double angleWiggle) const;
	void CalcGeneralGravity(FPOINT *gravity,double x,double y) const;

	void Move();
	bool IsParticleDead(const PARTICLE *particle) const;
	bool IsParticleDropped(int x,int y) const;
	void MoveAndSplitParticle(int x,int y,const PARTICLE *particle);
	void MoveParticle(int x,int y,const PARTICLE *particle
		,double angle,double angleWiggle,int generationLeft);
	void PutToBackMatrix(const PARTICLE *particle,const FPOINT &point
		,const FPOINT &vector,int generationLeft);
	const PARTICLE_INFO *ParticleInfo(const PARTICLE *particle) const;

	void CreateParticlePattern();
	void CreateBigPattern(unsigned char *buffer);
	void CreateSmallPatterns(unsigned char *smallPatterns
		,const unsigned char *bigPattern);
	void ReducePattern(unsigned char *smallPattern
		,const unsigned char *bigPattern);

	void Draw(Image *image);
	FPOINT ImageToMatrix(double x,double y) const;
	POINT ImageToMatrixInt(double x,double y) const;
	void MatrixToImage(FPOINT *imagePoint
		,const FPOINT &matrixPoint) const;
	void DrawParticle(const Image *image,int x,int y,PARTICLE *particle);

	PA_COLOR ParticleColor(const PARTICLE *particle) const;
	PA_COLOR MixPickedColor(const PARTICLE *particle,PA_COLOR color) const;
	PA_COLOR ParticleColor_Single(const PARTICLE_INFO *particleInfo) const;
	PA_COLOR ParticleColor_Gradation(const PARTICLE *particle
		,const PARTICLE_INFO *particleInfo) const;
	PA_COLOR ParticleColor_Cycle(const PARTICLE *particle
		,const PARTICLE_INFO *particleInfo) const;
	PA_COLOR ParticleColor_Random(const PARTICLE_INFO *particleInfo) const;
	
	PA_COLOR PickPixelColor(const Image *image,int x,int y) const;
	void AllocWork();
	void ClearWork(int mostTop,int rowCount);
	void MixImage(Image *image,int mostTop,int rowCount);
	DRAWWORK *GetWorkRowPtr(int y);
private:
	//`Ȃ
	Core(const Core &source);
	const Core &operator=(const Core &source);

	const std::auto_ptr<class CoreMember> m_m;
};

APP_END_NAMESPACE

#endif
