#include "prefix.h"

#include "delaycompress.h"
#include "ucharbuffer.h"

#include <zlib.h>

/*************************************************

	f[^\̒x k

**************************************************/

APP_BEGIN_NAMESPACE

#define OUTBUFFER_BYTES (1024*8)

class DelayCompressMember
{
public:
	DelayCompressMember(std::auto_ptr<Delay> argDelay,bool argDoesCompress)
		:delayPossess(argDelay)
		,doesCompress(argDoesCompress)
		,poolBytes(0)
		,pointer(0)
		,streamEnd(false)
	{
		delay=delayPossess.get();
	}
	DelayCompressMember(Delay *argDelay,bool argDoesCompress)
		:delay(argDelay)
		,doesCompress(argDoesCompress)
		,poolBytes(0)
		,pointer(0)
		,streamEnd(false)
	{
	}
	z_stream zstream;
	std::auto_ptr<Delay> delayPossess;
	Delay *delay;
	int poolBytes;
	int pointer;
	UCharBuffer outBuffer;
	bool doesCompress;
	bool streamEnd;
};

DelayCompress::DelayCompress(std::auto_ptr<Delay> delay,bool doesCompress)
	:m_m(new DelayCompressMember(delay,doesCompress))
{
	Initialize(doesCompress);
}

DelayCompress::DelayCompress(Delay *delay,bool doesCompress)
	:m_m(new DelayCompressMember(delay,doesCompress))
{
	Initialize(doesCompress);
}

void DelayCompress::Initialize(bool doesCompress)
{
	m_m->zstream.zalloc=(alloc_func)0;
	m_m->zstream.zfree=(free_func)0;
	m_m->zstream.opaque=(voidpf)0;
	if(doesCompress) {
		deflateInit(&m_m->zstream,Z_DEFAULT_COMPRESSION);
	} else {
		inflateInit(&m_m->zstream);
	}
}

DelayCompress::~DelayCompress() throw()
{
	if(m_m->doesCompress) {
		deflateEnd(&m_m->zstream);
	} else {
		inflateEnd(&m_m->zstream);
	}
}

bool DelayCompress::IsEOF() const
{
	return m_m->streamEnd;
}

int DelayCompress::PoolBytes() const
{
	return m_m->poolBytes-m_m->pointer;
}

const unsigned char *DelayCompress::Get()
{
	if(m_m->outBuffer.IsEmpty()) {
		m_m->outBuffer.Alloc(OUTBUFFER_BYTES);
	}

	if(PoolBytes()==0) {
		m_m->poolBytes=Read(m_m->outBuffer.GetW(),OUTBUFFER_BYTES);
		m_m->pointer=0;
	}
	return m_m->outBuffer.Get()+m_m->pointer;
}

void DelayCompress::Advance(int bytes)
{
	m_m->pointer+=bytes;
	if(m_m->pointer>=m_m->poolBytes) {
		m_m->pointer=m_m->poolBytes=0;
	}
}

int DelayCompress::Read(void *data,int bytes)
{
	m_m->zstream.next_out=static_cast<unsigned char*>(data);
	m_m->zstream.avail_out=bytes;

	int prevTotalOut=m_m->zstream.total_out;
	
	FOR(;;) {
		m_m->zstream.next_in=const_cast<unsigned char*>(m_m->delay->Get());
		int inBytes=m_m->delay->PoolBytes();
		m_m->zstream.avail_in=inBytes;
		if(m_m->delay->IsEOF()) {
			int result;
			if(m_m->doesCompress) {
				result=deflate(&m_m->zstream,Z_FINISH);
			} else {
				result=inflate(&m_m->zstream,Z_FINISH);
			}
			if(result==Z_STREAM_END) {
				m_m->streamEnd=true;
				break;
			}
		}

		if(m_m->doesCompress) {
			deflate(&m_m->zstream,Z_NO_FLUSH);
		} else {
			inflate(&m_m->zstream,Z_NO_FLUSH);
		}
		int usedBytes=inBytes - (m_m->zstream.avail_in);
		m_m->delay->Advance(usedBytes);

		if(m_m->zstream.avail_out==0) break;
	}
	return m_m->zstream.total_out-prevTotalOut;
}

void DelayCompress::Read(UCharBuffer *buffer)
{
	Delay::Read(buffer);
}

APP_END_NAMESPACE
