#include "prefix.h"

#include "delaydiff.h"

/*************************************************

	f[^\̒x 

**************************************************/

APP_BEGIN_NAMESPACE

#define OUTBUFFER_BYTES (1024*8)

class DelayDiffMember
{
public:
	DelayDiffMember(std::auto_ptr<Delay> left,std::auto_ptr<Delay> right,bool argDoesAdd)
		:leftDelayPossess(left)
		,rightDelayPossess(right)
		,doesAdd(argDoesAdd)
		,poolBytes(0)
		,pointer(0)
		,outBuffer(NULL)
		,isEOF(false)
		,hasDifference(false)
	{
		leftDelay=leftDelayPossess.get();
		rightDelay=rightDelayPossess.get();
	}
	DelayDiffMember(Delay *left,Delay *right,bool argDoesAdd)
		:leftDelay(left)
		,rightDelay(right)
		,doesAdd(argDoesAdd)
		,poolBytes(0)
		,pointer(0)
		,outBuffer(NULL)
		,isEOF(false)
		,hasDifference(false)
	{

	}
	std::auto_ptr<Delay> leftDelayPossess;
	std::auto_ptr<Delay> rightDelayPossess;
	Delay *leftDelay;
	Delay *rightDelay;
	int poolBytes;
	int pointer;
	unsigned char *outBuffer;
	bool isEOF;
	bool doesAdd;
	bool hasDifference;
};

DelayDiff::DelayDiff(std::auto_ptr<Delay> left
		,std::auto_ptr<Delay> right,bool doesAdd)

	:m_m(new DelayDiffMember(left,right,doesAdd))
{
}

DelayDiff::DelayDiff(Delay *left,Delay *right,bool doesAdd)
	:m_m(new DelayDiffMember(left,right,doesAdd))
{
}

DelayDiff::~DelayDiff() throw()
{
	delete[] m_m->outBuffer;
}

bool DelayDiff::IsEOF() const
{
	return (m_m->leftDelay->IsEOF() || m_m->rightDelay->IsEOF())
		&& PoolBytes()==0;
}

int DelayDiff::PoolBytes() const
{
	return m_m->poolBytes-m_m->pointer;
}

const unsigned char *DelayDiff::Get()
{
	if(m_m->outBuffer==NULL) {
		m_m->outBuffer=new unsigned char[OUTBUFFER_BYTES];
	}

	if(PoolBytes()==0) {
		m_m->poolBytes=Read(m_m->outBuffer,OUTBUFFER_BYTES);
		m_m->pointer=0;
	}
	return m_m->outBuffer+m_m->pointer;
}

void DelayDiff::Advance(int bytes)
{
	m_m->pointer+=bytes;
	if(m_m->pointer>=m_m->poolBytes) {
		m_m->pointer=m_m->poolBytes=0;
	}
}

int DelayDiff::Read(void *data,int bytes)
{
	unsigned char *dataPtr=static_cast<unsigned char *>(data);
	int written=0;
	while(written<bytes) {
		if(m_m->leftDelay->IsEOF() || m_m->rightDelay->IsEOF()) {
			break;
		}

		int sourceBytes=bytes-written;
		const unsigned char *leftPtr=m_m->leftDelay->Get();
		int leftPool=m_m->leftDelay->PoolBytes();
		if(leftPool<sourceBytes) {
			sourceBytes=leftPool;
		}
		const unsigned char *rightPtr=m_m->rightDelay->Get();
		int rightPool=m_m->rightDelay->PoolBytes();
		if(rightPool<sourceBytes) {
			sourceBytes=rightPool;
		}
		FOR(int i=0;i<sourceBytes;++i) {
			unsigned char lvalue=*leftPtr;
			++leftPtr;
			unsigned char rvalue=*rightPtr;
			++rightPtr;
			if(m_m->doesAdd) {
				lvalue+=rvalue;
			} else {
				lvalue-=rvalue;
				if(lvalue!=0) {
					m_m->hasDifference=true;
				}
			}
			*dataPtr=lvalue;
			++dataPtr;
		}
		written+=sourceBytes;
		m_m->leftDelay->Advance(sourceBytes);
		m_m->rightDelay->Advance(sourceBytes);
	}
	return written;
}

//Ⴂ邩
bool DelayDiff::HasDifference() const
{
	return m_m->hasDifference;
}

APP_END_NAMESPACE
