#include "prefix.h"

#include "dialog.h"
#include "main.h"
#include "exception.h"
#include "error.h"
#include "command.h"
#include "winmessage.h"
#include "updatecontrol.h"
#include "gdiobject.h"
#include "idtovalue.h"
#include "showerror.h"
#include "tooltip.h"
#include "modal.h"

#include "resdialog.h"

#include <windows.h>
#include <assert.h>
#include <exception>

/*************************************************

	_CAO{NX

**************************************************/

APP_BEGIN_NAMESPACE

//_CAÕbZ[W
BOOL CALLBACK AppDialogProc(HWND hwnd,UINT msg,WPARAM wparam,LPARAM lparam)
{
	Dialog *dialog=reinterpret_cast<Dialog *>
		(::GetWindowLong(hwnd,DWL_USER));

	BOOL result=FALSE;
	try {
		result=dialog->WindowProc(msg,wparam,lparam);
	} catch(std::exception &e) {
		ShowError(NULL,e);
	} 
	return result;
}

//_CAOIuWFNgɊtÕbZ[W
BOOL CALLBACK PreAttachDialogProc(HWND hwnd,UINT msg,WPARAM wparam,LPARAM lparam)
{
	if(msg==WM_INITDIALOG) {
		Dialog *dialog=reinterpret_cast<Dialog *>(lparam);
		dialog->Attach(hwnd);
		dialog->SetWindowLong(DWL_USER,LONG(lparam));
		dialog->SetWindowLong(DWL_DLGPROC,LONG(AppDialogProc));
		dialog->WindowProc(msg,wparam,lparam);
		return TRUE;
	}
	return FALSE;
}

class DialogMember
{
public:
	DialogMember(int id)
		:id(id)
		,isInitialized(false)
		,nowUpdating(false)
	{
	}

	ToolTip toolTip;
	const int id;	//_CAÕ\[XID
	bool isModal;	//[__CAO
	bool isInitialized;	//ꂽ
	bool nowUpdating;	//Rg[ƕϐŒlĂŒ
};

Dialog::Dialog(int id)
:m_m(new DialogMember(id))
{
	
}

Dialog::~Dialog() throw()
{
}

//[hX_CAO
void Dialog::Create(HWND parent)
{
	m_m->isModal=false;

	HWND hwnd=::CreateDialogParam(GetInstance(),MAKEINTRESOURCE(m_m->id)
		,parent,PreAttachDialogProc,LPARAM(this));
	if(hwnd==NULL) {
		throw Exception(APPERR_CANT_CREATE_WINDOW);
	}

	m_m->toolTip.Create(hwnd);
	m_m->toolTip.AddItems(hwnd);
}

//bZ[WU蕪
LRESULT Dialog::WindowProc(UINT msg,WPARAM wparam,LPARAM lparam)
{
	switch(msg) {
	case WM_COMMAND:
		PreOnCommand(wparam,lparam);
		break;

	case WM_INITDIALOG:
		PreOnInitDialog();
		break;

	case WM_DRAWITEM:
		OnDrawItem(wparam,(DRAWITEMSTRUCT*)lparam);
		break;

	case WM_NOTIFY:
		OnNotify((int)wparam,(NMHDR*)lparam);
		break;

	case APP_WM_CONTROL:
		OnControl(LOWORD(wparam),HIWORD(wparam),lparam);
		break;

	case APP_WM_UPDATE_COMMAND:
		OnUpdateCommand((UINT)wparam,(Command*)lparam);
		break;

	case WM_CLOSE:
		DestroyWindow();
		break;

	case  WM_DESTROY:
		OnDestroy();
		break;

	default:
		return FALSE;
	}
	return TRUE;
}

//OnInitDialogĂ
void Dialog::PreOnInitDialog()
{
	//OꂽEBhEj
	//̃IuWFNg̊֐try catchň͂ł͂Ȃ
	class WindowDestroyer
	{
	public:
		WindowDestroyer(HWND hwnd,bool isModal)
			:m_hwnd(hwnd)
			,m_isModal(isModal)
		{
		}
		~WindowDestroyer()
		{
			if(std::uncaught_exception()) {
				if(m_isModal) {
					::EndDialog(m_hwnd,IDCANCEL);
				} else {
					::DestroyWindow(m_hwnd);
				}
			}
		}
	private:
		HWND m_hwnd;
		bool m_isModal;
	} windowDestroyer(GetHWND(),m_m->isModal);

	OnInitDialog();
	m_m->isInitialized=true;
	UpdateData(UPDATE_CONTROL);
}

//ŏ̏
void Dialog::OnInitDialog()
{
}

//摜{^ɉ摜蓖Ă
void Dialog::SetButtonImage(GdiObject *holder,const IDToValueTable *table,int count)
{
	FOR(int i=0;i<count;++i) {
		HBITMAP hbitmap(::LoadBitmap(GetInstance()
			,MAKEINTRESOURCE(table[i].value)));
		holder[i].Attach(hbitmap);
		SendDlgItemMessage(table[i].id,BM_SETIMAGE
			,IMAGE_BITMAP,(LPARAM)(hbitmap));
	}
}

//R}hĂяo
void Dialog::PreOnCommand(WPARAM wparam,LPARAM lparam)
{
	if(!m_m->nowUpdating && IsInitialized()) {
		OnCommand(LOWORD(wparam),HIWORD(wparam),(HWND)lparam);
	}
}

//R}h̏
void Dialog::OnCommand(int id,int,HWND)
{
	switch(id) {
	case IDOK:
		OnOK();
		break;
	case IDC_APPLY:
		OnApply();
		break;

	case IDCANCEL:
		OnCancel();
		break;
	}
}

//R}hGUI̍XV
void Dialog::OnUpdateCommand(UINT,Command *)
{

}

void Dialog::OnControl(int id,int code,LPARAM lparam)
{

}

//[__CAOJ
int Dialog::DoModal(HWND parent)
{
	m_m->isModal=true;

	Modal modal;
	modal.Disable();

	int result=::DialogBoxParam(GetInstance(),MAKEINTRESOURCE(m_m->id)
		,parent,PreAttachDialogProc,LPARAM(this));

	if(result==-1) {
		throw Exception(APPERR_CANT_CREATE_WINDOW);
	}
	return result;
}


//OK{^
void Dialog::OnOK()
{
	UpdateData(UPDATE_VAR);
	if(m_m->isModal) {
		if(Apply()) {
			::EndDialog(GetHWND(),IDOK);
		}
	}
}

//Kp{^
void Dialog::OnApply()
{
	UpdateData(UPDATE_VAR);
	if(m_m->isModal) {
		Apply();
	}
}

//LZ{^
void Dialog::OnCancel()
{
	if(m_m->isModal) {
		::EndDialog(GetHWND(),IDCANCEL);
	} 
}

//Kp
bool Dialog::Apply()
{
	return true;
}

//Rg[̎gpEs̐؂ւ
void Dialog::EnableItem(UINT itemID,bool isEnabled)
{
	HWND item(GetDlgItem(itemID));
	assert(item!=NULL);

	::EnableWindow(item,isEnabled);
}

//ϐ̏ςł邩
bool Dialog::IsInitialized() const
{
	return m_m->isInitialized;
}

//Rg[̍XV
void Dialog::UpdateData(bool toControl)
{
	if(!m_m->nowUpdating) {
		m_m->nowUpdating=true;
		UpdateControl updater(this,toControl);
		DataExchange(&updater);
		m_m->nowUpdating=false;
	}
}

//I[i[`
void Dialog::OnDrawItem(int itemID,DRAWITEMSTRUCT *drawItemStruct)
{
	SendDlgItemMessage(itemID,APP_WM_REFLECT_DRAWITEM,(WPARAM)drawItemStruct,0);
}

bool Dialog::DestroyWindow()
{
	if(m_m->isModal) {
		::EndDialog(GetHWND(),IDCANCEL);
	} else {
		return Window::DestroyWindow();
	}
	return true;
}

void Dialog::OnDestroy()
{
	Window::OnDestroy();
}

void Dialog::OnNotify(int,NMHDR *nmhdr)
{
	switch(nmhdr->code) {
	case TTN_NEEDTEXT:
		SetToolTipText((TOOLTIPTEXT*)nmhdr);
		break;
	}
}

APP_END_NAMESPACE
