#include "prefix.h"

#include "filedialog.h"
#include "string.h"
#include "verify.h"
#include "modal.h"

#include <windows.h>
#include <commdlg.h>
#include <string.h>
#include <tchar.h>

/*************************************************

	t@C_CAO

**************************************************/

APP_BEGIN_NAMESPACE

class FileDialogMember
{
public:
	String filename;
	String caption;
	String filter;
	OPENFILENAME openParam;
};

FileDialog::FileDialog(UINT captionID,UINT filterID
	,const _TCHAR * filename,const _TCHAR * defaultExt,DWORD flags
	,HWND parentWnd)
:m_m(new FileDialogMember)
{
	memset(&m_m->openParam,0,sizeof(OPENFILENAME));
	m_m->openParam.lStructSize=sizeof(OPENFILENAME);
	m_m->openParam.hwndOwner=parentWnd;

	if(filterID!=0) {
		VERIFY(m_m->filter.LoadString(filterID));

		_TCHAR * ptr=m_m->filter.GetW();
		FOR(;;) {
			ptr=_tcschr(ptr,L'|');
			if(ptr==NULL) break;
			*ptr=0;
			++ptr;
		}

		m_m->openParam.lpstrFilter=m_m->filter.Get();
	} else {
		m_m->openParam.lpstrFilter=NULL;
	}

	m_m->filename=filename;
	m_m->filename.Alloc(_MAX_PATH);
	m_m->openParam.lpstrFile=m_m->filename.GetW();
	m_m->openParam.nMaxFile=_MAX_PATH;

	VERIFY(m_m->caption.LoadString(captionID));
	m_m->openParam.lpstrTitle=m_m->caption.Get();
	m_m->openParam.Flags=flags;
	m_m->openParam.lpstrDefExt=defaultExt;
}

FileDialog::~FileDialog() throw()
{

}

bool FileDialog::OpenFileName()
{
	Modal modal;
	modal.Disable();

	m_m->openParam.Flags|=OFN_EXPLORER|OFN_FILEMUSTEXIST
		|OFN_PATHMUSTEXIST|OFN_HIDEREADONLY;
	return ::GetOpenFileName(&m_m->openParam)!=FALSE;	
}

bool FileDialog::SaveFileName()
{
	Modal modal;
	modal.Disable();

	m_m->openParam.Flags=OFN_EXPLORER|OFN_HIDEREADONLY;
	return ::GetSaveFileName(&m_m->openParam)!=FALSE;
}

const _TCHAR * FileDialog::GetFileName() const
{
	return m_m->filename.Get();
}

APP_END_NAMESPACE

