#include "prefix.h"

#include "frame.h"
#include "view.h"
#include "command.h"
#include "winmessage.h"
#include "string.h"
#include "winman.h"
#include "tooltip.h"

#include "resstring.h"
#include "rescommand.h"

#include <windows.h>
#include <commctrl.h>
#include <assert.h>

/*************************************************

	OgEBhENX

**************************************************/

APP_BEGIN_NAMESPACE

Frame::Frame()
:m_view(NULL)
,m_lastStatusTextID(0)
{
}

Frame::~Frame() throw()
{

}

//r[EBhEIuWFNgL
void Frame::AttachView(View *view)
{
	assert(view!=NULL && m_view.get()==NULL);
	std::auto_ptr<View> viewPtr(view);
	m_view=viewPtr;
}

//EBhE
void Frame::Create(const RECT &rect,HWND parentWindow)
{
	Window::Create(NULL,WS_OVERLAPPEDWINDOW,rect,parentWindow);
	m_view->Create(GetHWND());
	m_view->SetWindowLong(GWL_ID,ID_VIEW);
}

//bZ[WU蕪
LRESULT Frame::WindowProc(UINT msg,WPARAM wparam,LPARAM lparam)
{
	switch(msg) {
	case WM_NCDESTROY:
		OnNcDestroy();
		break;

	case WM_CLOSE:
		OnClose();
		break;

	case WM_SIZE:
		OnSize((UINT)wparam,LOWORD(lparam),HIWORD(lparam));
		break;

	case WM_INITMENUPOPUP:
		OnInitMenuPopup(reinterpret_cast<HMENU>(wparam)
			,LOWORD(lparam)
			,HIWORD(lparam)!=FALSE);
		break;

	case WM_MENUSELECT:
		OnMenuSelect(LOWORD(wparam),HIWORD(wparam),HMENU(lparam));
		break;

	case WM_KEYDOWN:
		OnKeyDown(wparam,lparam);
		break;

	case WM_KEYUP:
		OnKeyUp(wparam,lparam);
		break;

	case WM_ACTIVATEAPP:
		OnActivateApp(wparam,lparam);
		break;

	case WM_NOTIFY:
		OnNotify((int)wparam,(NMHDR*)lparam);
		break;

	case APP_WM_UPDATE_WINDOW:
		OnUpdateWindow();
		break;

	default:
		return Window::WindowProc(msg,wparam,lparam);
	}
	return 0;
}

//EBhEj
void Frame::OnNcDestroy()
{
	ClosedMainFrame(this);
}

//EBhȆ傫ύXꂽ
void Frame::OnSize(UINT,int,int)
{
	AdjustChildWindows();
}

//EBhE̔zu
void Frame::AdjustChildWindows()
{
	RECT rect;
	GetClientRect(&rect);

	HWND statusBar=GetDlgItem(ID_STATUSBAR);
	if(statusBar!=NULL && ::IsWindowVisible(statusBar)) {
		::SendMessage(statusBar,WM_SIZE,0,0);
		RECT statusBarRect;
		::GetWindowRect(statusBar,&statusBarRect);
		rect.bottom-=Height(statusBarRect);
	}

	HWND toolBar=GetDlgItem(ID_TOOLBAR);
	if(toolBar!=NULL && ::IsWindowVisible(toolBar)) {
		::SendMessage(toolBar,WM_SIZE,0,0);
		RECT toolBarRect;
		::GetWindowRect(toolBar,&toolBarRect);
		rect.top+=Height(toolBarRect);
	}

	m_view->SetWindowPos(0,rect.left,rect.top,Width(rect),Height(rect),0);
}

//r[EBhEIuWFNg𓾂
View *Frame::GetView()
{
	return m_view.get();
}

//R}h
void Frame::OnCommand(int id,int notifyCode,HWND hwnd)
{
	switch(id) {
	case ID_APP_EXIT:
		OnClose();
		break;

	default:
		m_view->SendMessage(WM_COMMAND,MAKELONG(id,notifyCode),(LPARAM)hwnd);
		break;
	}
}

//j[̕\XV
void Frame::OnInitMenuPopup(HMENU hmenu,int,bool isSystemMenu)
{
	if(isSystemMenu) return;

	const int menuCount=::GetMenuItemCount(hmenu);

	for(int i=0;i<menuCount;++i) {
		Command command;
		command.SetMenu(hmenu,i);
		int commandID=::GetMenuItemID(hmenu,i);
		OnUpdateCommand(commandID,&command);
	}
}

//c[o[ȂǎōXVȂGUI̕\XV
void Frame::OnUpdateWindow()
{
	HWND toolBar=GetDlgItem(ID_TOOLBAR);
	if(toolBar==NULL) return;

	const int count=::SendMessage(toolBar,TB_BUTTONCOUNT,0,0);
	for(int i=0;i<count;++i) {
		TBBUTTON tbbutton;
		::SendMessage(toolBar,TB_GETBUTTON,i
			,reinterpret_cast<LPARAM>(&tbbutton));
		int id=tbbutton.idCommand;
		Command command;
		command.SetToolBar(toolBar,id);
		OnUpdateCommand(id,&command);
	}
}

//R}hGUI̍XV
void Frame::OnUpdateCommand(int id,Command *command)
{
	GetView()->SendMessage(APP_WM_UPDATE_COMMAND
			,(WPARAM)id,(LPARAM)command);
}

//Xe[^Xo[͂̕̕\
void Frame::SetStatusText(int textID)
{
	if(textID==m_lastStatusTextID) return;
	m_lastStatusTextID=textID;

	String text;
	if(textID!=0) {
		String res;
		if(!res.LoadString(textID)) {
			return;
		}
		text.SingleLine(res);
	}

	HWND statusBar=GetDlgItem(ID_STATUSBAR);
	if(statusBar!=NULL) {
		::SendMessage(statusBar,WM_SETTEXT,0,(LPARAM)(text.Get()));
	}
}

//j[J[\wƂ̏
void Frame::OnMenuSelect(int item,UINT flag,HMENU hmenu)
{
	int messageID=item;
	if((flag&MF_POPUP)!=0
		|| (flag==0xffff && hmenu==NULL)) {
		messageID=IDS_IDLE;
	}
	SetStatusText(messageID);
}

//L[
void Frame::OnKeyDown(WPARAM wparam,LPARAM lparam)
{
	GetView()->SendMessage(WM_KEYDOWN,wparam,lparam);
}

//L[𗣂
void Frame::OnKeyUp(WPARAM wparam,LPARAM lparam)
{
	GetView()->SendMessage(WM_KEYUP,wparam,lparam);
}

//ANeBuEBhE؂ւ
void Frame::OnActivateApp(WPARAM wparam,LPARAM lparam)
{
	bool on=(wparam!=FALSE);
	GetView()->SendMessage(WM_ACTIVATEAPP,on,lparam);
}

//EBhEO
void Frame::OnClose()
{
	GetView()->SendMessage(APP_WM_FILE_CLOSE,0,0);
	DestroyWindow();
}

//NotifybZ[W̏
void Frame::OnNotify(int,NMHDR *nmhdr)
{
	switch(nmhdr->code) {
	case TTN_NEEDTEXT:
		SetToolTipText((TOOLTIPTEXT*)nmhdr);
		break;
	}
}

//ĂȂƂ
bool Frame::OnIdle()
{
	SetStatusText(IDS_IDLE);
	View *view=GetView();
	if(view!=NULL) {
		return view->OnIdle();
	}
	return false;
}

APP_END_NAMESPACE
