#include "prefix.h"

#include "generalpanel.h"
#include "updatecontrol.h"
#include "fpoint.h"
#include "anglecontrol.h"
#include "volumecontrol.h"
#include "generalstatus.h"
#include "winmessage.h"
#include "idtovalue.h"
#include "gdiobject.h"
#include "limitdialog.h"
#include "registry.h"

#include "resdialog.h"
#include "resbitmap.h"

#include <windows.h>
#include <tchar.h>

/*************************************************

	Ŝ̏Ԃ̃pl

**************************************************/

APP_BEGIN_NAMESPACE

static const IDToValueTable s_buttons[]={
	{IDC_GENERAL_PARALLEL,IDB_GENERAL_PARALLEL},
	{IDC_GENERAL_RADIAL,IDB_GENERAL_RADIAL},
	{IDC_DETAIL_GENERAL_GRAVITY,IDB_DETAIL},
};
#define BUTTON_COUNT (sizeof(s_buttons)/sizeof(IDToValueTable))

class GeneralPanelMember
{
public:
	AngleControl gravityAngleControl;
	VolumeControl gravityControl;
	VolumeControl livelyControl;

	GdiObject buttonImage[BUTTON_COUNT];
	double gravityLimit;	//d͂̃{[Rg[̏
};

GeneralPanel::GeneralPanel()
:Panel(_T("Panel\\General"),IDD_GENERAL)
,m_m(new GeneralPanelMember)
{
}

GeneralPanel::~GeneralPanel() throw()
{

}

void GeneralPanel::ExchangeSettings(Registry *registry)
{
	Panel::ExchangeSettings(registry);

	registry->OpenSubKey(_T("Panel\\General\\Limit"));
	registry->Exchange(_T("Gravity"),&(m_m->gravityLimit),5);
}

//
void GeneralPanel::OnInitDialog()
{
	Panel::OnInitDialog();

	SetButtonImage(m_m->buttonImage,s_buttons,BUTTON_COUNT);

	m_m->gravityAngleControl.SubclassDialogItem(
		IDC_GENERAL_GRAVITY_ANGLE,this);
	m_m->gravityControl.SubclassDialogItem(
		IDC_GENERAL_GRAVITY,this);
	m_m->livelyControl.SubclassDialogItem(
		IDC_GENERAL_LIVELY,this);

	m_m->livelyControl.SetShowMode(VOLUME_SHOW_PERCENT);
}

void GeneralPanel::OnCommand(int id,int notifyCode,HWND hwnd)
{
	switch(id) {
	case IDC_GENERAL_PARALLEL:
	case IDC_GENERAL_RADIAL:
		if(notifyCode==BN_CLICKED) {
			UpdateData(UPDATE_VAR);
		}
		break;

	case IDC_DETAIL_GENERAL_GRAVITY:
		OnDetailButton(id);
		break;

	default:
		Panel::OnCommand(id,notifyCode,hwnd);
		break;
	}
}

void GeneralPanel::OnControl(int id,int code,LPARAM lparam)
{
	switch(id) {
	case IDC_GENERAL_GRAVITY_ANGLE:
		if(code==ANGLE_CHANGED) {
			UpdateData(UPDATE_VAR);
		}
		break;

	case IDC_GENERAL_GRAVITY:
	case IDC_GENERAL_LIVELY:
		if(code==VOLUME_CHANGED) {
			UpdateData(UPDATE_VAR);
		}
		break;

	default:
		Panel::OnControl(id,code,lparam);
		break;
	}
}

static const IDToValueTable s_gravityTypeTable[]=
{
	{IDC_GENERAL_PARALLEL,GRAVITY_PARALLEL},
	{IDC_GENERAL_RADIAL,GRAVITY_RADIAL},

	{0,0}
};

//Rg[ƕϐ̃f[^\̍XV
void GeneralPanel::DataExchange(UpdateControl *updater)
{
	GENERAL_STATUS generalStatus;
	GetGeneralStatus(&generalStatus);

	updater->Vector(IDC_GENERAL_GRAVITY,IDC_GENERAL_GRAVITY_ANGLE,&(generalStatus.gravity));
	updater->Volume(IDC_GENERAL_LIVELY,&(generalStatus.lively));
	updater->RadioButton(s_gravityTypeTable,&(generalStatus.gravityType));

	if(updater->ToControl()) {
		m_m->gravityControl.SetRange(0,m_m->gravityLimit);
	}

	if(!updater->ToControl()) {
		SetGeneralStatus(generalStatus);
	}
}

//ڍאݒ{^
void GeneralPanel::OnDetailButton(int)
{
	LimitDialog limitDialog;
	limitDialog.SetVolumeLimit(m_m->gravityLimit,0.001);
	limitDialog.DoModal(GetHWND());
	m_m->gravityLimit=limitDialog.GetVolumeLimit();
	UpdateData(UPDATE_CONTROL);
}

APP_END_NAMESPACE
