#include "prefix.h"

#include "history.h"
#include "historywriter.h"
#include "historyreader.h"
#include "historyitem.h"
#include "options.h"

#include "ptrdeque.h"

/*************************************************

	

**************************************************/

APP_BEGIN_NAMESPACE

//񑀍앪
typedef PtrDeque<HistoryItem> HistoryRecord;

typedef PtrDeque<HistoryRecord> HistoryList;

class HistoryMember
{
public:
	HistoryMember()
		:writeNest(0)
		,recordIndex(0)
		,readItemIndex(0)
		,isReading(false)
	{
	}
	~HistoryMember()
	{
	}
	HistoryList list;

	std::auto_ptr<HistoryRecord> currentRecord;
	int writeNest;	//̓o^̓q
	int recordIndex;	//݂̈ʒu
	int readItemIndex;
	bool isReading;	//蒼⌳ɖ߂ĂŒ
	bool undo;
};

History::History() throw()
:m_m(new HistoryMember)
{
}

History::~History() throw()
{

}

//A̗̓̓o^n߂
bool History::BeginWrite()
{
	if(GetOptions()->historyMax==0) {
		return false;
	}

	assert(m_m->writeNest>=0);
	++m_m->writeNest;

	if(!m_m->isReading && m_m->writeNest==1) {
		assert(m_m->currentRecord.get()==NULL);
		APP_RESET_AUTO_PTR(m_m->currentRecord,HistoryRecord,new HistoryRecord);
	}
	return true;
}

//A̗̓̓o^I
void History::EndWrite()
{
	assert(GetOptions()->historyMax>0);
	
	--m_m->writeNest;
	assert(m_m->writeNest>=0);

	if(!m_m->isReading
		&& m_m->writeNest==0) {
		if(!m_m->currentRecord->IsEmpty()) {
			m_m->list.EraseAfter(m_m->recordIndex);

			int max=GetOptions()->historyMax;
			int eraseCount=Count() - max + 1;
			if(eraseCount>0) {
				m_m->list.EraseBefore(eraseCount);
				m_m->recordIndex-=eraseCount;
			}

			m_m->list.Add(m_m->currentRecord.get());
			m_m->currentRecord.release();
			++m_m->recordIndex;
		} else {
			delete m_m->currentRecord.release();
		}
	}
}

//ڂǉ
void History::PushItem(HistoryItem *item)
{
	std::auto_ptr<HistoryItem> itemHolder(item);
	assert(m_m->currentRecord.get()!=NULL);
	m_m->currentRecord->Add(itemHolder.get());
	itemHolder.release();
}

//̓o^𒆎~
void History::CancelWrite() throw()
{
	--m_m->writeNest;
	assert(m_m->writeNest>=0);

	delete m_m->currentRecord.release();
}

//̓ǂݎ̎n
void History::BeginRead(bool undo)
{
	assert(m_m->writeNest==0 && !m_m->isReading);
	assert(!undo || m_m->recordIndex>0);
	assert(undo || m_m->recordIndex<Count());

	m_m->isReading=true;
	m_m->readItemIndex=0;
	m_m->undo=undo;
}

void History::EndRead()
{
	assert(m_m->writeNest==0 && m_m->isReading);
	if(m_m->undo) {
		--m_m->recordIndex;
	} else {
		++m_m->recordIndex;
	}
	m_m->isReading=false;
}

void History::CancelRead() throw()
{
	if(m_m->undo) {
		++m_m->recordIndex;
	}
	m_m->isReading=false;	
}

//̍ڂ𓾂
const HistoryItem *History::NextItem()
{
	const HistoryItem *item=NULL;
	if(m_m->undo) {
		const HistoryRecord *record;
		record=m_m->list.At(m_m->recordIndex-1);
		if(m_m->readItemIndex<record->Count()) {
			item=record->RevAt(m_m->readItemIndex);
			++m_m->readItemIndex;
		}
	} else {
		const HistoryRecord *record;
		record=m_m->list.At(m_m->recordIndex);
		if(m_m->readItemIndex<record->Count()) {
			item=record->At(m_m->readItemIndex);
			++m_m->readItemIndex;
		}
	}
	return item;
}

//ڐ
int History::Count() const
{
	return m_m->list.Count();
}

//ɖ߂\
bool History::CanUndo() const
{
	return m_m->recordIndex>0;
}

//蒼\
bool History::CanRedo() const
{
	return m_m->recordIndex<Count();
}

//ݒύX
void History::OnChangeOptions()
{
	assert(m_m->currentRecord.get()==NULL);
	assert(!m_m->isReading);

	int max=GetOptions()->historyMax;
	if(max<Count()) {
		m_m->list.EraseAfter(m_m->recordIndex);
		int eraseCount=Count() - max;
		if(eraseCount>0) {
			m_m->list.EraseBefore(eraseCount);
			m_m->recordIndex-=eraseCount;
		}
	}
}

APP_END_NAMESPACE
