#include "prefix.h"

#include "particleinfolist.h"
#include "particleinfo.h"

#include "trace.h"

#include <string.h>
#include <limits.h>
#include <assert.h>
#include <memory>
#include <tchar.h>

/*************************************************

	q̑̔z

**************************************************/

APP_BEGIN_NAMESPACE

#ifndef NDEBUG
//#define DEBUG_DUMP
#endif

//vf
struct PARTICLE_INFO_ITEM : public PARTICLE_INFO
{
	union {
		int refer;	//̃e[uQƂĂ鐔
		int nextEmpty;	//󂫃Xg̗̎vf̔ԍ
	};
	int branch;	//e番򂵂Ărbgʒu
	int next;	//eŕ򂵂Ăvf
	int child;	//̗vf番򂵂Ăvf
};

//xɊmۂvf
#define ALLOC_BLOCK_COUNT 64

#define NIL	-1

ParticleInfoList::ParticleInfoList()
:m_emptyList(NIL)
,m_bufferCount(0)
,m_root(NIL)
#ifndef NDEBUG
,m_count(0)
#endif
{
}

ParticleInfoList::~ParticleInfoList() throw()
{

}


//̌
struct SEARCHRESULT
{
	const PARTICLE_INFO_ITEM *list;
	int previous;	//ڑ
	int current;	//l̗vf
	int branch;	//̂rbg
	bool isChild;	//previous̎qƂĐڑ邩
};

//q̎ނȂ
bool ParticleInfoList::IsEmpty() const
{
	return m_root==NIL;	
}

//
int ParticleInfoList::AddType(const PARTICLE_INFO &particleInfo)
{
#ifdef DEBUG_DUMP
	TRACE(_T("AddType"));
#endif

	PARTICLE_INFO_ITEM newItem;
	ClearItem(&newItem);
	newItem.CopyParticleInfo(particleInfo);

	if(IsEmpty()) {
		m_root=0;
		m_array.Increase(ALLOC_BLOCK_COUNT);
		m_array[0]=newItem;
		m_array[0].refer=1;
		m_bufferCount=1;
		m_emptyList=NIL;

#ifdef DEBUG_DUMP
		DumpArray();
#endif
#ifndef NDEBUG
		++m_count;
#endif
		return 0;
	}

	SEARCHRESULT result;
	Search(&result,newItem);
	if(result.current!=NIL) {
		assert(0<=result.current && result.current<m_bufferCount);
		++m_array[result.current].refer;

#ifdef DEBUG_DUMP
		TRACE(_T("find"));
#endif
		return result.current;
	}

	if(m_emptyList==NIL) {
		IncreaseArray();
	}
	int currentIndex=m_emptyList;
	PARTICLE_INFO_ITEM *current;
	current=&(m_array[currentIndex]);
	m_emptyList=current->nextEmpty;
	*current=newItem;
	current->refer=1;

	assert(result.previous!=NIL);
	PARTICLE_INFO_ITEM *previous=&(m_array[result.previous]);

	assert(result.branch!=NIL);

	current->branch=result.branch;

	if(result.isChild) {
		current->next=previous->child;
		previous->child=currentIndex;
	} else {
		current->next=previous->next;
		previous->next=currentIndex;
	}

#ifdef DEBUG_DUMP
	DumpArray();
#endif
#ifndef NDEBUG
	++m_count;
#endif

	return currentIndex;
}

//z̗vf
void ParticleInfoList::ClearItem(PARTICLE_INFO_ITEM *item) const
{
	memset(item,0,sizeof(PARTICLE_INFO_ITEM));
	item->refer=0;
	item->branch=NIL;
	item->next=NIL;
	item->child=NIL;
}

//vf𑝂₷
void ParticleInfoList::IncreaseArray()
{
	int overBlock=(m_bufferCount+1) + (ALLOC_BLOCK_COUNT-1);
	int block=overBlock - (overBlock%ALLOC_BLOCK_COUNT);
	m_array.Increase(block);
	ClearItem(&(m_array[m_bufferCount]));
	m_array[m_bufferCount].nextEmpty=m_emptyList;
	m_emptyList=m_bufferCount;
	++m_bufferCount;
}

//p𑝂₷
void ParticleInfoList::IncreaseReferenceCount(int index)
{
#ifndef NDEBUG
	FOR(int i=m_emptyList;i!=NIL;i=m_array[i].nextEmpty) {
		assert(i!=index);
	}
#endif

#ifdef DEBUG_DUMP
	TRACE(_T("Increase Reference Count %d"),index);
#endif
	++m_array[index].refer;
}

//p炷
void ParticleInfoList::DecreaseReferenceCount(int index)
{
#ifndef NDEBUG
	FOR(int i=m_emptyList;i!=NIL;i=m_array[i].nextEmpty) {
		assert(i!=index);
	}
#endif

#ifdef DEBUG_DUMP
	TRACE(_T("Decrease Reference Count %d"),index);
#endif
	assert(m_array[index].refer>0);
	--m_array[index].refer;
	Clean(index);
}

//pĂȂ΍폜
void ParticleInfoList::Clean(int index)
{
	assert(m_array[index].refer>=0);
	if(m_array[index].refer==0) {
		Delete(index);
	}
}

//vf폜
void ParticleInfoList::Delete(int index)
{
#ifdef DEBUG_DUMP
	TRACE(_T("Delete %d"),index);
#endif

	assert(0<=index && index<m_bufferCount);
	assert(m_array[index].refer==0);

	SEARCHRESULT result;
	Search(&result,m_array[index]);

	int current=result.current;
	assert(0<=current && current<m_bufferCount);

	bool isChild=result.isChild;

	m_array[current].nextEmpty=m_emptyList;

	m_emptyList=current;

#ifndef NDEBUG
	--m_count;
#endif

	int previous=result.previous;
	int *previousLink=NULL;
	if(previous!=NIL) {
		if(isChild) {
			previousLink=&(m_array[previous].child);
		} else {
			previousLink=&(m_array[previous].next);
		}
	} else {
		previousLink=&m_root;
	}

	if(m_array[current].child==NIL) {
		*previousLink=m_array[current].next;

#ifdef DEBUG_DUMP
		DumpArray();
#endif
		return;
	}
	
	int child=m_array[current].child;
	int lastChild=child;
	int secondLast=NIL;
	while(m_array[lastChild].next!=NIL) {
		secondLast=lastChild;
		lastChild=m_array[lastChild].next;
	}

	*previousLink=lastChild;

	m_array[lastChild].branch=m_array[current].branch;
	m_array[lastChild].next=m_array[current].next;
	if(lastChild!=child) {
		int lastChildChild=m_array[lastChild].child;
		m_array[lastChild].child=child;
		m_array[secondLast].next=lastChildChild;
	}

#ifdef DEBUG_DUMP
	DumpArray();
#endif
}

//𓾂
const PARTICLE_INFO *ParticleInfoList::Get(int index)
{
#ifdef DEBUG_DUMP
	TRACE(_T("Get %d"),index);
#endif

	assert(0<=index && index<=m_bufferCount);

	const PARTICLE_INFO_ITEM *item=&(m_array[index]);
	assert(item!=NULL);

	return static_cast<const PARTICLE_INFO*>(item);
}

//
void ParticleInfoList::Search(SEARCHRESULT *result,const PARTICLE_INFO_ITEM &item) const
{
	int current=m_root;
	int previous=NIL;

	result->current=NIL;
	result->isChild=true;
	int branch=-1;
	FOR(;;) {
		branch=BranchBit(m_array[current],item,branch+1);

		//
		if(branch==NIL) {
			result->current=current;
			result->previous=previous;
			return;
		}

		result->branch=branch;
		result->isChild=true;

		//qȂ
		if(m_array[current].child==NIL) {
			result->previous=current;
			return;
		}
	
		previous=current;
		current=m_array[previous].child;
		FOR(;;) {
			//݂̎q̎O
			if(branch<m_array[current].branch) {
				result->previous=previous;
				return;
			}

			//ꏊŕ򂷂q
			if(branch==m_array[current].branch) {
				break;
			}

			result->isChild=false;

			//Ȃ
			if(m_array[current].next==NIL) {
				result->previous=current;
				return;
			}

			//̎q
			previous=current;
			current=m_array[current].next;
		}
	}
}

//ŏɈႢ̏orbgʒu
//NILȂ瓙
int ParticleInfoList::BranchBit(const PARTICLE_INFO_ITEM &itemA,const PARTICLE_INFO_ITEM &itemB,int startBit) const
{
	const unsigned char *a
		=reinterpret_cast<const unsigned char *>(&itemA);
	const unsigned char *b
		=reinterpret_cast<const unsigned char *>(&itemB);

	int index=startBit / CHAR_BIT;
	int bit=startBit % CHAR_BIT;
	FOR(;index<sizeof(PARTICLE_INFO);++index) {
		unsigned char diff=a[index] ^ b[index];
		diff>>=bit;
		if(diff>0) {
			FOR(;bit<CHAR_BIT;++bit) {
				if(diff&1) {
					return index*CHAR_BIT+bit;
				}
				diff>>=1;
			}
		}
		bit=0;
	}
	return NIL;
}

//Sĉ
void ParticleInfoList::Free()
{
	m_array.Free();
	m_bufferCount=0;
	m_emptyList=NIL;
}

#ifndef NDEBUG
void ParticleInfoList::DumpArray()
{
	TRACE(_T("Dump count %d root %d emptyTop %d")
		,m_bufferCount,m_root,m_emptyList);

	FOR(int i=0;i<m_bufferCount;++i) {
		TRACE(_T("[%d] "
			"refer %d "
			"branch %d "
			"next %d "
			"child %d")
			,i
			,m_array[i].refer
			,m_array[i].branch
			,m_array[i].next
			,m_array[i].child);
	}
}
#endif

APP_END_NAMESPACE
