#include "prefix.h"

#include "spraypanel.h"

#include "updatecontrol.h"
#include "fpoint.h"
#include "anglecontrol.h"
#include "volumecontrol.h"
#include "tool.h"
#include "spray.h"
#include "spraystyle.h"
#include "winman.h"
#include "winmessage.h"
#include "string.h"
#include "verify.h"
#include "gdiobject.h"
#include "registry.h"
#include "idtovalue.h"

#include "limitdialog.h"
#include "velocitydetail.h"
#include "gravitydetail.h"
#include "repulsiondetail.h"
#include "viscositydetail.h"

#include "spindialog.h"
#include "splitdialog.h"

#include "resdialog.h"
#include "resstring.h"
#include "resbitmap.h"

#include <windows.h>
#include <tchar.h>

/*************************************************

	Xv[̐ݒ̃pl

**************************************************/

APP_BEGIN_NAMESPACE

static const IDToValueTable s_buttons[]={
	{IDC_DETAIL_REPULSION,IDB_DETAIL},
	{IDC_DETAIL_GRAVITY,IDB_DETAIL},
	{IDC_DETAIL_VISCOSITY,IDB_DETAIL},
	{IDC_DETAIL_VELOCITY,IDB_DETAIL},
	{IDC_DETAIL_LIMITED_SPEED,IDB_DETAIL},
	{IDC_DETAIL_COUNT,IDB_DETAIL},
	{IDC_DETAIL_LIFE,IDB_DETAIL},
	{IDC_DETAIL_SPRAY_SIZE,IDB_DETAIL},
	{IDC_DETAIL_THROWING,IDB_DETAIL},
	{IDC_SPIN,IDB_SPIN},
	{IDC_SPLIT,IDB_SPLIT},
};
#define BUTTON_COUNT (sizeof(s_buttons)/sizeof(IDToValueTable))

class SprayPanelMember
{
public:
	VolumeControl volumeControl[SPRAYVOLUME_TYPE_COUNT];	//ʃRg[
	AngleControl velocityAngleControl;	//̊pxRg[

	double upperLimit[SPRAYVOLUME_TYPE_COUNT];	//ʃRg[̏l

	GdiObject buttonImage[BUTTON_COUNT];
};

SprayPanel::SprayPanel()
:Panel(_T("Panel\\Spray"),IDD_SPRAYSTYLE)
,m_m(new SprayPanelMember)
{

}

SprayPanel::~SprayPanel() throw()
{
}

void SprayPanel::ExchangeSettings(Registry *registry)
{
	Panel::ExchangeSettings(registry);

	registry->OpenSubKey(_T("Panel\\Spray\\Limit"));

	FOR(int i=0;i<SPRAYVOLUME_TYPE_COUNT;++i) {
		if(SprayVolumeType(i)->editableLimit) {
			registry->Exchange(SprayVolumeType(i)->name
				,&(m_m->upperLimit[i]),SprayVolumeType(i)->initLimit);
			if(m_m->upperLimit[i]<SprayVolumeType(i)->lowerLimit) {
				m_m->upperLimit[i]=SprayVolumeType(i)->lowerLimit;
			}
		}
	}
}

//
void SprayPanel::OnInitDialog()
{
	Panel::OnInitDialog();

	SetButtonImage(m_m->buttonImage,s_buttons,BUTTON_COUNT);

	FOR(int i=0;i<SPRAYVOLUME_TYPE_COUNT;++i) {
		int id=SprayVolumeType(i)->controlID;
		m_m->volumeControl[i].SubclassDialogItem(id,this);
		int showMode=SprayVolumeType(i)->showMode;
		m_m->volumeControl[i].SetShowMode(showMode);
	}

	//l̏
#define INIT_LIMIT(a1,a_idName,a3,a_editableLimit,a_initLimit,a6,a7,a8)	\
	if(!a_editableLimit) {	\
		m_m->upperLimit[SPRAYVOLUME_##a_idName]=a_initLimit;	\
	}

	SPRAYVOLUME_DO_FUNC(INIT_LIMIT);

	m_m->velocityAngleControl.SubclassDialogItem(
		IDC_VELOCITY_ANGLE,this);
}

LRESULT SprayPanel::WindowProc(UINT msg,WPARAM wparam,LPARAM lparam)
{
	switch(msg) {
	case APP_WM_CHANGE_SPRAY_STYLE:
		OnChangeStyle();
		break;

	case APP_WM_SET_TOOL:
		OnSetTool(wparam);
		break;

	default:
		return Panel::WindowProc(msg,wparam,lparam);
	}
	return 0;
}

void SprayPanel::OnCommand(int id,int notifyCode,HWND hwnd)
{
	//ڍאݒ{^ID
#define DETAIL_CASE(a1,a_idName,a3,a4,a5,a6,a7,a8)	\
	case IDC_DETAIL_##a_idName:

	switch(id) {
	SPRAYVOLUME_DO_FUNC(DETAIL_CASE)
		OnDetailButton(id);
		break;

	case IDC_USE_LIMITED_SPEED:
		if(notifyCode==BN_CLICKED) {
			UpdateData(UPDATE_VAR);
			UpdateData(UPDATE_CONTROL);
		}
		break;

	case IDC_SPIN:
		if(notifyCode==BN_CLICKED) {
			OnSpinButton();
		}
		break;

	case IDC_SPLIT:
		if(notifyCode==BN_CLICKED) {
			OnSplitButton();
		}
		break;

	default:
		Panel::OnCommand(id,notifyCode,hwnd);
		break;
	}
}

void SprayPanel::OnControl(int id,int code,LPARAM lparam)
{
	switch(id) {
	//ʃRg[ID
#define VOLUME_CASE(a1,a_idName,a3,a4,a5,a6,a7,a8)	\
	case IDC_##a_idName:

	SPRAYVOLUME_DO_FUNC(VOLUME_CASE)
		if(code==VOLUME_DRAW_VALUE) {
			OnDrawVolumeValue(id,(DRAWVOLUME*)lparam);
		} else if(code==VOLUME_CHANGED) {
			UpdateData(UPDATE_VAR);
		}
		break;

	case IDC_VELOCITY_ANGLE:
		if(code==ANGLE_CHANGED) {
			UpdateData(UPDATE_VAR);
		}
		break;

	default:
		Panel::OnControl(id,code,lparam);
		break;
	}
}


//Rg[ƕϐ̃f[^\̍XV
void SprayPanel::DataExchange(UpdateControl *updater)
{
	SPRAYSTYLE spray;
	GetSprayStyle(&spray);

	updater->Volume(IDC_REPULSION,&(spray.repulsion));
	updater->Volume(IDC_GRAVITY,&(spray.gravity));
	updater->Volume(IDC_BRAKE,&(spray.brake));
	updater->Volume(IDC_VISCOSITY,&(spray.viscosity));
	updater->Volume(IDC_LIMITED_SPEED,&(spray.limitedSpeed));
	updater->Volume(IDC_COUNT,&(spray.count));
	updater->Volume(IDC_SPRAY_SIZE,&(spray.size));
	updater->Volume(IDC_LIFE,&(spray.life));
	updater->Volume(IDC_THROWING,&(spray.throwing));

	updater->Vector(IDC_VELOCITY,IDC_VELOCITY_ANGLE,&(spray.velocity));
	updater->Check(IDC_USE_LIMITED_SPEED,&(spray.useLimitedSpeed));
	updater->Enable(IDC_LIMITED_SPEED,spray.useLimitedSpeed);

	updater->Check(IDC_SPIN,&(spray.useSpin));
	updater->Check(IDC_SPLIT,&(spray.useSplit));

	if(!updater->ToControl()) {
		SetSprayStyle(spray);
	} else {
		FOR(int i=0;i<SPRAYVOLUME_TYPE_COUNT;++i) {
			int lowerLimit=SprayVolumeType(i)->lowerLimit;
			m_m->volumeControl[i].SetRange(lowerLimit
				,m_m->upperLimit[i]);
		}
	}
}

//VolumeControl̐l\
void SprayPanel::OnDrawVolumeValue(int itemID,DRAWVOLUME *drawVolume)
{
	String string;

	double value=drawVolume->volume;
	int format;
	switch(itemID) {
	case IDC_COUNT:
		format=IDS_FORMAT_COUNT;
		break;

	default:
		assert(false);
	}

	string.Format(format,value);
	VERIFY(::DrawText(drawVolume->hdc,string.Get(),string.Length()
		,&(drawVolume->rect),DT_RIGHT|DT_SINGLELINE|DT_VCENTER)>0);
}

//ŕύX
void SprayPanel::OnChangeStyle()
{
	UpdateData(UPDATE_CONTROL);
}

//c[؂ւ
void SprayPanel::OnSetTool(int)
{
	UpdateData(UPDATE_CONTROL);
}

//ڍ׃{^ID獀ڂ̔ԍ𓾂
#define DETAIL_TO_NUMBER(a1,a_idName,a3,a4,a5,a6,a7,a8)	\
	{IDC_DETAIL_##a_idName,SPRAYVOLUME_##a_idName},

const static IDToValueTable s_detailButton[]=
{
	SPRAYVOLUME_DO_FUNC(DETAIL_TO_NUMBER)

	{0,0}
};

//ڔԍ_CAOIuWFNg
#define DETAIL_DIALOG(a1,a_idName,a3,a4,a5,a6,a_detailDialog,a8)	\
	case SPRAYVOLUME_##a_idName:	\
		APP_RESET_AUTO_PTR(detailDialog,LimitDialog,new a_detailDialog);	\
		break;

//ڍאݒ{^
void SprayPanel::OnDetailButton(int id)
{
	int number=IDToValue(s_detailButton,id);	
	double volumeLimit=m_m->upperLimit[number];
	double lowerLimit=SprayVolumeType(number)->lowerLimit;

	std::auto_ptr<LimitDialog> detailDialog;
	switch(number) {
		SPRAYVOLUME_DO_FUNC(DETAIL_DIALOG)
	}

	detailDialog->SetVolumeLimit(volumeLimit,lowerLimit);
	detailDialog->DoModal(GetHWND());
	m_m->upperLimit[number]=detailDialog->GetVolumeLimit();
	UpdateData(UPDATE_CONTROL);
}

//]̃{^B
void SprayPanel::OnSpinButton()
{
	SpinDialog spinDialog;
	spinDialog.DoModal(GetHWND());
	UpdateData(UPDATE_CONTROL);
}

//̃{^
void SprayPanel::OnSplitButton()
{
	SplitDialog splitDialog;
	splitDialog.DoModal(GetHWND());
	UpdateData(UPDATE_CONTROL);
}

APP_END_NAMESPACE
