#include "prefix.h"

#include "main.h"
#include "string.h"
#include "exception.h"
#include "error.h"
#include "verify.h"
#include "buffer.h"

#include <windows.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <tchar.h>

/*************************************************

	NX

**************************************************/

APP_BEGIN_NAMESPACE

//̒obt@̒zĂȂ
#ifdef NDEBUG
#define LENGTH_TEST()	\
if(!IsEmpty()) {	\
	assert(_tcslen(Get())+1<=Count());	\
}
#else
#define LENGTH_TEST()
#endif

class StringMember
{
public:
	StringMember()
	{
	}
	~StringMember() throw()
	{
	}
	Buffer<_TCHAR> buffer;
};

String::String()
:m_m(new StringMember)
{
}

String::String(const String &source)
:m_m(new StringMember)
{
	*this=source;
}

String::String(const char *text)
:m_m(new StringMember)
{
	*this=text;
}

String::String(const wchar_t *text)
:m_m(new StringMember)
{
	*this=text;
}

String::~String() throw()
{
}

const String &String::operator=(const String &source)
{
	m_m->buffer=source.m_m->buffer;
	return *this;
}

//ݒ肷
const String & String::operator=(const char *text)
{
	if(text!=NULL) {
#ifdef _UNICODE
		int length=mbstowcs(NULL,text,strlen(text)+1)+1;
		Alloc(length);
		_wcsnset(GetW(),0,length);
		mbstowcs(GetW(),text,strlen(text)+1);
#else
		Alloc(strlen(text)+1);
		_mbscpy(reinterpret_cast<unsigned char*>(GetW())
			,reinterpret_cast<const unsigned char*>(text));
#endif
	} else {
		Free();
	}
	LENGTH_TEST();
	return *this;
}

const String & String::operator=(const wchar_t *text)
{
	if(text!=NULL) {
#ifdef _UNICODE
		Alloc(wcslen(text)+1);
		wcscpy(GetW(),text);
#else
		int length=wcstombs(NULL,text,0)+1;
		Alloc(length);
		strnset(GetW(),0,length);
		wcstombs(GetW(),text,length);
#endif
	} else {
		Free();
	}
	LENGTH_TEST();
	return *this;
}

//Oꂽobt@J
//̃NX̃IuWFNg̊֐try catchň͂ł͂Ȃ
class StringCrasher
{
public:
	StringCrasher(String *string)
		:m_string(string)
	{
	}
	~StringCrasher()
	{
		if(std::uncaught_exception()) {
			m_string->Free();
		}
	}
private:
	String *m_string;
};

//\[Xǂݍ
bool String::LoadString(int id)
{
	StringCrasher stringCrasher(this);

	const int block=256*sizeof(_TCHAR);
	int size=0;
	int loadedBytes;
	do {
		size+=block;
		Alloc(size);

		loadedBytes=::LoadString(GetInstance(),id,GetW(),size);
		if(loadedBytes==0 && ::GetLastError()!=ERROR_SUCCESS) {
			Free();
			return false;
		}
	} while(loadedBytes>=size*sizeof(_TCHAR));

	Shorten();
	return true;
}

//當
void VStringFormat(String *string,const _TCHAR * format,va_list args)
{
	StringCrasher stringCrasher(string);

	const int block=256;
	int size=0;
	do {
		size+=block;
		string->Alloc(size);
	} while(_vsntprintf(string->GetW(),size,format,args)<0);
	string->Shorten();
}

//printfł̏ id͂\[X
void String::Format(int id,...)
{
	String format;
	VERIFY(format.LoadString(id));

	va_list args;
	va_start(args,id);

	VStringFormat(this,format.Get(),args);

	va_end(args);
}

//printfł̏
void String::Format(const _TCHAR * format,...)
{
	va_list args;
	va_start(args,format);

	VStringFormat(this,format,args);

	va_end(args);
}

//VXe͂̕o
void String::LoadSystem(DWORD apiErrorCode,...)
{
	struct HLocalHolder
	{
		HLocalHolder(HLOCAL hlocal)
			:m_hlocal(hlocal)
		{
		}
		~HLocalHolder()
		{
			::LocalFree(m_hlocal);
		}
	private:
		HLOCAL m_hlocal;
	};

	va_list args;
	va_start(args,apiErrorCode);

	_TCHAR * buffer;
	if(!::FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM
			|FORMAT_MESSAGE_ALLOCATE_BUFFER
		,NULL
		,apiErrorCode
		,MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT)
		,(_TCHAR *)&buffer
		,0
		,&args)) {
		throw Exception(APPERR_API,::GetLastError());
	}
	HLocalHolder hlocalHolder(buffer);
	*this=buffer;

	va_end(args);
}

//̃|C^𓾂
const _TCHAR * String::Get() const
{
	LENGTH_TEST();
	if(IsEmpty()) {
		return _T("");
	}
	return m_m->buffer.Get();
}

_TCHAR * String::GetW()
{
	LENGTH_TEST();
	return m_m->buffer.GetW();
}

//obt@̊m
void String::Alloc(int size)
{
	LENGTH_TEST();
	int oldSize=Length();

	m_m->buffer.Realloc(size);
	if(size>oldSize) {
		memset(GetW()+oldSize*sizeof(_TCHAR),0,(size-oldSize)*sizeof(_TCHAR));
	}
}

//obt@𕶎̖[܂łɐ؂l߂
void String::Shorten()
{
	LENGTH_TEST();
	if(!IsEmpty()) {
		Alloc(Length()+1);
	}
}

//s̎n߂̃AhX
const _TCHAR * String::StartOfLine(int line) const
{
	LENGTH_TEST();
	const _TCHAR * start=Get();
	FOR(int i=0;i<line;++i) {
		const _TCHAR * end=_tcschr(start,L'\n');
		if(end==NULL) {
			return NULL;
		}
		start=end+1;
	}
	return start;
}

//so
bool String::SingleLine(const String &source,int line)
{
	const _TCHAR * start=source.StartOfLine(line);
	if(start==NULL) {
		return false;
	}

	const _TCHAR * end=_tcschr(start,L'\n');

	int length;
	if(end!=NULL) {
		length=static_cast<int>(end-start);
	} else {
		length=_tcslen(start);
	}

	Alloc(length+1);

	memset(GetW(),0,(length+1)*sizeof(_TCHAR));
	_tcsncpy(GetW(),start,length);
	LENGTH_TEST();
	return true;
}

//̒
int String::Length() const
{
	LENGTH_TEST();
	if(IsEmpty()) {
		return 0;
	}
	return _tcslen(Get());
}

//r
int String::Compare(const _TCHAR * text) const
{
	LENGTH_TEST();
	return _tcscmp(Get(),text);
}

//݂̌̕ɂȂ
void String::Cat(const _TCHAR * text)
{
	Alloc(Length()+_tcslen(text)+1);
	_tcscat(GetW(),text);
	LENGTH_TEST();
}

void String::Free() throw()
{
	m_m->buffer.Free();
}

bool String::IsEmpty() const
{
	return m_m->buffer.IsEmpty();
}

#ifdef _UNICODE
int String::MultiByteLength() const
{
	return wcstombs(NULL,Get(),0);
}

void String::ToMultiByte(char *mbText,int length)
{
	strnset(mbText,0,length);
	wcstombs(mbText,Get(),length);
}

#else

int String::UnicodeLength() const
{
	return mbstowcs(NULL,Get(),Length()+1);
}

void String::ToUnicode(wchar_t *wcText,int length)
{
	_wcsnset(wcText,0,length);
	VERIFY(mbstowcs(wcText,Get(),Length()+1)<length);
}
#endif

APP_END_NAMESPACE
