#include "prefix.h"

#include "tool.h"
#include "toolbase.h"
#include "event.h"
#include "exception.h"
#include "error.h"
#include "winman.h"
#include "winmessage.h"
#include "registry.h"
#include "string.h"

#include "toolpaint.h"
#include "toolerase.h"
#include "toolblur.h"

#include <assert.h>

/*************************************************

	c[Ǘ

**************************************************/

APP_BEGIN_NAMESPACE

TOOLTYPE s_toolType[TOOL_COUNT]={
	//O	F		Z	Xv[
	{_T("Paint"),true	,true	,true},	//h
	{_T("Erase"),false	,true	,true},	//
	{_T("Blur")	,false	,true	,true},	//ɂ
};

const TOOLTYPE *ToolType(int tool)
{
	assert(0<=tool && tool<TOOL_COUNT);
	return &(s_toolType[tool]);
}

static int s_tool;	//c[̔ԍ
static ToolBase *s_toolObject=NULL;	//c[̃IuWFNg

ToolBase *CreateObject(int tool);

static void ExchangeSettings(Registry *registry);

//AvP[V̎n߂̏
void AppInit_Tool(Registry *registry)
{
	ExchangeSettings(registry);
	s_toolObject=CreateObject(s_tool);
}

//ݒ̕ۑ
void AppSave_Tool(Registry *registry)
{
	ExchangeSettings(registry);
}

//AvP[V̏I
void AppQuit_Tool() throw()
{
	delete s_toolObject;
	s_toolObject=NULL;
}

static void ExchangeSettings(Registry *registry)
{
	String toolName;
	if(!registry->IsReading()) {
		toolName=s_toolType[s_tool].name;
	}
	registry->OpenSubKey(NULL);
	registry->Exchange(_T("Tool"),&toolName,s_toolType[s_tool].name);

	if(registry->IsReading()) {
		s_tool=TOOL_PAINT;
		FOR(int i=0;i<TOOL_COUNT;++i) {
			if(_tcscmp(toolName.Get(),s_toolType[i].name)==0) {
				s_tool=i;
				break;
			}
		}
	}
}

//c[IuWFNg؂ւ
void SetTool(int tool)
{
	if(IsToolOperating()) {
		throw Exception(APPERR_TOOL_OPERATING);
	}

	delete s_toolObject;
	s_toolObject=NULL;
	s_toolObject=CreateObject(tool);
	s_tool=tool;

	SendAllWindowsMessage(APP_WM_SET_TOOL,(WPARAM)s_tool,0);
}

//c[IuWFNg쐬
ToolBase *CreateObject(int tool)
{
	ToolBase *object=NULL;
	switch(tool) {
	case TOOL_PAINT:
		object=static_cast<ToolBase *>(new ToolPaint);
		break;

	case TOOL_ERASE:
		object=static_cast<ToolBase *>(new ToolErase);
		break;

	case TOOL_BLUR:
		object=static_cast<ToolBase *>(new ToolBlur);
		break;

	default:
		assert(false);
	}
	return object;
}

//Cxg
void ToolEntry(EVENT_PARAM *eventParam)
{
	s_toolObject->Entry(eventParam);
}

//J[\ǐՂ邩
bool IsToolCapturing()
{
	return s_toolObject->IsCapturing();
}

//c[s
bool IsToolOperating()
{
	return s_toolObject->IsOperating();
}

int GetTool()
{
	return s_tool;
}

APP_END_NAMESPACE
