#include "prefix.h"

#include "toolfreehand.h"
#include "core.h"
#include "event.h"
#include "spray.h"
#include "spraystyle.h"
#include "pi.h"
#include "color.h"
#include "fpoint.h"
#include "cellsize.h"

#include <math.h>
#include <stdlib.h>

/*************************************************

	菑̃c[̊{NX

**************************************************/

APP_BEGIN_NAMESPACE

enum PHASE
{
	PHASE_IDLE,
	PHASE_PUT,
};

class ToolFreeHandMember
{
public:
	ToolFreeHandMember(int argTool)
		:tool(argTool)
		,phase(PHASE_IDLE)
		,infoNumber(-1)
#ifndef NDEBUG
		,particleReference(false)
#endif
	{
	}

	SPRAYSTYLE sprayStyle;
	FPOINT lastPoint;	//Ōɓ_łW
	FPOINT lastThrowing;	//Ō̃J[\̐̕
	int infoNumber;	//q̑̔ԍ
	const int tool;	//hĂc[̎
	int phase;	//

	int times;	//{^Ẵ^[
#ifndef NDEBUG
	bool particleReference;
#endif
};

ToolFreeHand::ToolFreeHand(int tool)
:m_m(new ToolFreeHandMember(tool))
{
}

ToolFreeHand::~ToolFreeHand() throw()
{
}

bool ToolFreeHand::IsCapturing() const
{
	return m_m->phase!=PHASE_IDLE;
}

//s
bool ToolFreeHand::IsOperating() const
{
	return m_m->phase!=PHASE_IDLE;
}

//Cxg	ԂƂɐU蕪
void ToolFreeHand::Event(EVENT_PARAM *eventParam)
{
	switch(m_m->phase) {
	case PHASE_IDLE:
		EventIdle(eventParam);
		break;

	case PHASE_PUT:
		EventPut(eventParam);
		break;
	}
}

//ĂȂԂł̃Cxg
void ToolFreeHand::EventIdle(EVENT_PARAM *eventParam)
{
	switch(eventParam->event) {
	case EVENT_LEFTDOWN:
		Start(eventParam);
		break;
	}
}

//quJn
void ToolFreeHand::Start(EVENT_PARAM *eventParam)
{
	GetSprayStyle(&(m_m->sprayStyle));
	GetColorInfo(&(m_m->sprayStyle.colorInfo));

	PrepareSprayStyleToUse(&(m_m->sprayStyle));

	Core *core=eventParam->core;
	if(m_m->infoNumber!=-1) {
		core->DecreaseReferenceParticleInfo(m_m->infoNumber);
	}

	m_m->infoNumber=core->AddParticleInfo(m_m->sprayStyle);
#ifndef NDEBUG
	assert(!m_m->particleReference);
	m_m->particleReference=true;
#endif

	m_m->times=0;

	FirstPut(eventParam);
}

//ŏɗqu
void ToolFreeHand::FirstPut(EVENT_PARAM *eventParam)
{
	m_m->phase=PHASE_PUT;

	m_m->lastThrowing.x=m_m->lastThrowing.y=0;
	Put(eventParam,eventParam->point,m_m->lastThrowing,0);
	m_m->lastPoint=eventParam->point;
}

//quĂԂł̃Cxg
void ToolFreeHand::EventPut(EVENT_PARAM *eventParam)
{
	switch(eventParam->event) {
	case EVENT_CANCEL:
	case EVENT_LEFTUP:
		Cancel(eventParam);
		break;

	case EVENT_TURN:
		Turn(eventParam);
		break;
	}
}

//f
void ToolFreeHand::Cancel(EVENT_PARAM *eventParam)
{
	Core *core=eventParam->core;
	core->DecreaseReferenceParticleInfo(m_m->infoNumber);
	m_m->infoNumber=-1;
#ifndef NDEBUG
	assert(m_m->particleReference);
	m_m->particleReference=false;
#endif
	m_m->phase=PHASE_IDLE;
}

//^[
void ToolFreeHand::Turn(EVENT_PARAM *eventParam)
{
	const int sprayCount=m_m->sprayStyle.count;
	assert(sprayCount>0);

	FPOINT curPoint=eventParam->point;

	FPOINT point=m_m->lastPoint;
	double dx=(curPoint.x-m_m->lastPoint.x)/sprayCount;
	double dy=(curPoint.y-m_m->lastPoint.y)/sprayCount;

	FPOINT throwing=m_m->lastThrowing;
	FPOINT curThrowing=curPoint-m_m->lastPoint;

	double dtx=(curThrowing.x-throwing.x)/sprayCount;
	double dty=(curThrowing.y-throwing.y)/sprayCount;

	FOR(int i=0;i<sprayCount;++i) {
		point.x+=dx;
		point.y+=dy;
		throwing.x+=dtx;
		throwing.y+=dty;
		Put(eventParam,point,throwing,i);
	}

	m_m->lastPoint=point;
	m_m->lastThrowing=curThrowing;
	++m_m->times;
}

//֐𔲂Ƃɗq̏̎QƉ񐔂炷
class DecreaseReference
{
public:
	DecreaseReference()
		:m_core(NULL)
	{
	}
	~DecreaseReference()
	{
		if(m_core!=NULL) {
			m_core->DecreaseReferenceParticleInfo(m_infoNumber);
		}			
	}
	void Set(Core *core,int infoNumber)
	{
		m_core=core;
		m_infoNumber=infoNumber;
	}
private:
	Core *m_core;
	int m_infoNumber;
};

//q𗎂Ƃ
void ToolFreeHand::Put(EVENT_PARAM *eventParam,const FPOINT &point
	,const FPOINT &throwing,int number)
{
	FPOINT velocity=m_m->sprayStyle.velocity;
	
	//J[\̐
	double throwingRate=m_m->sprayStyle.throwing;
	if(throwingRate>0) {
		velocity.x+=throwing.x*throwingRate;
		velocity.y+=throwing.y*throwingRate;
	}

	//]
	if(m_m->sprayStyle.velocityWidth>0) {
		double velocityAngle=SprayAngle(number);
		FPOINT oldVelocity=velocity;
		velocity.x=oldVelocity.x*cos(velocityAngle) - oldVelocity.y*sin(velocityAngle);
		velocity.y=oldVelocity.x*sin(velocityAngle) + oldVelocity.y*cos(velocityAngle);
	}

	//h炬
	double wiggle=m_m->sprayStyle.velocityWiggle;
	if(wiggle>0) {
		double rate=rand()*wiggle/RAND_MAX;
		velocity.x-=velocity.x*rate;
		velocity.y-=velocity.y*rate;
	}

	PUT_PARAM putParam;
	putParam.velocity=velocity;

	DecreaseReference decreaseReference;

	int infoNumber=m_m->infoNumber;

	if(m_m->sprayStyle.colorInfo.coloring==COLORING_RANDOM) {
		//΂΂̒F̏ꍇ
		PARTICLE_INFO particleInfo;
		particleInfo.CopyParticleInfo(m_m->sprayStyle);
		particleInfo.colorInfo.color[0]
			=RandomColor(particleInfo.colorInfo);
		Core *core=eventParam->core;
		infoNumber=core->AddParticleInfo(particleInfo);
		decreaseReference.Set(core,infoNumber);
	}
	putParam.particleInfo=infoNumber;

	int size=m_m->sprayStyle.size;

	double r=(double)rand()*size/(RAND_MAX*2);
	double angle=(rand()*PI*2)/RAND_MAX;

	double x=cos(angle)*r;
	double y=sin(angle)*r;

	putParam.point.x=point.x+x;
	putParam.point.y=point.y+y;
	putParam.image=eventParam->image;
	putParam.splitGenerationsMax=m_m->sprayStyle.splitGenerationsMax;
	eventParam->core->Put(putParam);
}

//̊pxɉpx
double ToolFreeHand::SprayAngle(int number)
{
	double rotation=m_m->sprayStyle.velocityRotationSpeed;
	double width=m_m->sprayStyle.velocityWidth;
	double sum=rotation*m_m->times
		+rotation*number/m_m->sprayStyle.count
		+width/2;

	double angle;
	double wipe;
	switch(m_m->sprayStyle.velocityMode) {
	case VELOCITY_LEFT_SPIN:
		angle=fmod(sum,width)-width/2;
		break;

	case VELOCITY_RIGHT_SPIN:
		angle=-(fmod(sum,width)-width/2);
		break;

	case VELOCITY_WIPER:
		wipe=fmod(sum,width*2);
		if(wipe>width) {
			wipe=width*2-wipe;
		}
		angle=wipe-width/2;
		break;

	case VELOCITY_RANDOM:
		angle=rand()*width/(RAND_MAX+1)-width/2;
		break;

	default:
		assert(false);
	}
	return angle*2*PI/360;
}

//΂΂̐F
PA_COLOR ToolFreeHand::RandomColor(const COLOR_INFO &colorInfo) const
{
	PA_COLOR color(colorInfo.color[FORECOLOR]);
	PA_COLOR alterColor(colorInfo.color[BACKCOLOR]);

	int random=rand();

	FOR(int c=0;c<PA_COLOR_BYTES;++c) {
		int mix=(color.channel[c]*(RAND_MAX-random)
			+alterColor.channel[c]*random)/RAND_MAX;
		color.channel[c]=static_cast<unsigned char>(mix);
	}
	return color;
}

APP_END_NAMESPACE
