#include "prefix.h"

#include "updatecontrol.h"
#include "dialog.h"
#include "string.h"
#include "anglecontrol.h"
#include "volumecontrol.h"
#include "fpoint.h"
#include "idtovalue.h"
#include "verify.h"

#include <windows.h>
#include <commctrl.h>
#include <math.h>
#include <assert.h>

/*************************************************

	_CAÕRg[̍XV

**************************************************/

APP_BEGIN_NAMESPACE

UpdateControl::UpdateControl(Dialog *dialog,bool toControl)
:m_dialog(dialog)
,m_toControl(toControl)
{
	assert(dialog!=NULL);
}

UpdateControl::~UpdateControl() throw()
{

}

//Rg[XV邩
bool UpdateControl::ToControl() const
{
	return m_toControl;
}

//Rg[̎gpEs̐؂ւ
void UpdateControl::Enable(int itemID,bool isEnabled)
{
	if(m_toControl) {
		m_dialog->EnableItem(itemID,isEnabled);	
	}
}

//Rg[̍̕XV
void UpdateControl::Edit(int itemID,String *string)
{
	if(m_toControl) {
		m_dialog->SetDlgItemText(itemID,string->Get());
	} else {
		m_dialog->GetDlgItemText(itemID,string);
	}
}

//Rg[̐l̍XV
void UpdateControl::Edit(int itemID,int *value)
{
	if(m_toControl) {
		m_dialog->SetDlgItemInt(itemID,*value);
	} else {
		BOOL succeed;
		int itemValue=m_dialog->GetDlgItemInt(itemID,&succeed);
		if(succeed) {
			*value=itemValue;
		}
	}
}

//Rg[̏̍XV
void UpdateControl::Edit(int itemID,double *value)
{
	if(m_toControl) {
		String string;
		string.Format(_T("%g"),*value);
		m_dialog->SetDlgItemText(itemID,string.Get());
	} else {
		String string;
		m_dialog->GetDlgItemText(itemID,&string);
		*value=_tcstod(string.Get(),NULL);
	}
}

//`FbN{^Rg[̍XV
void UpdateControl::Check(int itemID,bool *isChecked)
{
	if(m_toControl) {
		UINT state;
		if(*isChecked) {
			state=BST_CHECKED;
		} else {
			state=BST_UNCHECKED;
		}
		m_dialog->CheckDlgButton(itemID,state);
	} else {
		UINT state=m_dialog->IsDlgButtonChecked(itemID);		
		if(state==BST_CHECKED) {
			*isChecked=true;
		} else {
			*isChecked=false;
		}
	}
}

//AngleControl̍XV
void UpdateControl::Angle(int itemID,double *angle)
{
	HWND item=m_dialog->GetDlgItem(itemID);
	AngleControl *control=reinterpret_cast<AngleControl*>
		(GetWindowObject(item));
	if(control==NULL) return;

	if(m_toControl) {
		control->SetAngle(*angle);
	} else {
		*angle=control->GetAngle();
	}
}

//VolumeControl̍XV
void UpdateControl::Volume(int itemID,double *volume)
{
	HWND item=m_dialog->GetDlgItem(itemID);
	VolumeControl *control=reinterpret_cast<VolumeControl*>
		(GetWindowObject(item));
	if(control==NULL) return;

	if(m_toControl) {
		control->SetVolume(*volume);
	} else {
		*volume=control->GetVolume();
	}
}

void UpdateControl::Volume(int itemID,int *volume)
{
	double volumeDouble=*volume;
	Volume(itemID,&volumeDouble);
	*volume=static_cast<int>(volumeDouble);
}

//ƒ
void UpdateControl::Vector(int volumeID,int angleID,FPOINT *vector)
{
	double angle,length;
	if(m_toControl) {
		angle=APP_ATAN_XY(vector->x,-vector->y);
		length=hypot(vector->x,vector->y);
	}
	Volume(volumeID,&length);
	Angle(angleID,&angle);
	if(!m_toControl) {
		vector->x=cos(angle)*length;
		vector->y=-sin(angle)*length;
	}
}

//XC_[
void UpdateControl::Slider(int itemID,int *position)
{
	HWND item=m_dialog->GetDlgItem(itemID);
	assert(item!=NULL);
	if(m_toControl) {
		::SendMessage(item,TBM_SETPOS,TRUE,*position);
	} else {
		*position=::SendMessage(item,TBM_GETPOS,0,0);
	}
}

//WI{^
void UpdateControl::RadioButton(const IDToValueTable *idTable,int *selection)
{
	HWND dialog=m_dialog->GetHWND();
	assert(dialog!=NULL);
	FOR(int i=0;i<idTable[i].id!=0;++i) {
		int id=idTable[i].id;
		assert(m_dialog->GetDlgItem(id)!=NULL);
		if(m_toControl) {
			bool on=(idTable[i].value==*selection);
			UINT state=on?BST_CHECKED:BST_UNCHECKED;
			VERIFY(::CheckDlgButton(dialog,id,state));
		} else {
			UINT state=::IsDlgButtonChecked(dialog,id);
			if(state==BST_CHECKED) {
				*selection=idTable[i].value;
				break;
			}
		}
	}
}

APP_END_NAMESPACE
