#include "prefix.h"

#include "frame.h"
#include "view.h"
#include "document.h"
#include "winmessage.h"

#include <assert.h>

/*************************************************

	r[EBhE{NX

**************************************************/

APP_BEGIN_NAMESPACE

View::View()
:m_document(NULL)
{
	
}

View::~View() throw()
{
	if(m_document!=NULL) {
		delete m_document;
	}
}

//r[EBhE
void View::Create(HWND frameWindow)
{
	RECT rect={CW_USEDEFAULT,CW_USEDEFAULT
		,CW_USEDEFAULT,CW_USEDEFAULT};

	Window::Create(NULL,WS_VISIBLE|WS_CHILD|WS_HSCROLL|WS_VSCROLL,rect,frameWindow);

	GetDocument()->OnNewDocument();
	OnInitialUpdate();
}

//bZ[WU蕪
LRESULT View::WindowProc(UINT msg,WPARAM wparam,LPARAM lparam)
{
	switch(msg) {
	case WM_HSCROLL:
		OnHScroll((int)LOWORD(wparam),(short int)HIWORD(wparam),HWND(wparam));
		break;

	case WM_VSCROLL:
		OnVScroll((int)LOWORD(wparam),(short int)HIWORD(wparam),HWND(wparam));
		break;

	case WM_SIZE:
		OnSize((UINT)wparam,LOWORD(lparam),HIWORD(lparam));
		break;

	case APP_WM_UPDATE_COMMAND:
		OnUpdateCommand((UINT)wparam,(Command*)lparam);
		break;

	case APP_WM_FILE_CLOSE:
		OnFileClose();
		break;

	default:
		return Window::WindowProc(msg,wparam,lparam);
	}
	return 0;
}

//t[EBhẼ|C^
Frame *View::GetFrame()
{
	return m_frame;
}

//t[EBhEIuWFNgݒ肷BL͂Ȃ
void View::SetFrame(Frame *frame)
{
	m_frame=frame;
}

//IuWFNgL
void View::AttachDocument(Document *document)
{
	assert(document!=NULL && m_document==NULL);
	m_document=document;
}

#define SCROLLLINESIZE 16
#define SCROLLPAGESIZE 64

//r[̃XN[ݒ
void View::SetScrollSize(SIZE size)
{
	m_scrollSize=size;

	SCROLLINFO info;
	info.cbSize=sizeof(info);
	info.fMask=SIF_POS|SIF_PAGE|SIF_RANGE;
	info.nPage=SCROLLPAGESIZE;
	info.nPos=0;
	info.nMin=0;
	info.nMax=0;
	SetScrollInfo(SB_HORZ,&info,FALSE);
	SetScrollInfo(SB_VERT,&info,FALSE);

	OnSize(0,0,0);
}

//̃XN[
void View::OnHScroll(int scrollCode,short int pos,HWND)
{
	OnScroll(scrollCode,pos,SB_HORZ);
}

//c̃XN[
void View::OnVScroll(int scrollCode,short int pos,HWND)
{
	OnScroll(scrollCode,pos,SB_VERT);
}

//XN[̏
void View::OnScroll(int scrollCode,short int pos,int bar)
{
	SCROLLINFO scrollInfo;
	scrollInfo.cbSize=sizeof(SCROLLINFO);
	scrollInfo.fMask=SIF_POS|SIF_RANGE|SIF_PAGE;
	GetScrollInfo(bar,&scrollInfo);

	int scrollMax=scrollInfo.nMax;

	int oldPos=scrollInfo.nPos;
	int newPos;
	switch(scrollCode) {
	default:
		return;

	case SB_BOTTOM:
		newPos=scrollMax;
		break;

	case SB_LINEDOWN:
		newPos=oldPos+SCROLLLINESIZE;
		break;

	case SB_LINEUP:
		newPos=oldPos-SCROLLLINESIZE;
		break;

	case SB_PAGEDOWN:
		newPos=oldPos+scrollInfo.nPage;
		break;

	case SB_PAGEUP:
		newPos=oldPos-scrollInfo.nPage;
		break;
		
	case SB_THUMBPOSITION:
	case SB_THUMBTRACK:
		newPos=pos;
		break;

	case SB_TOP:
		newPos=0;
		break;
	}

	RECT clientRect;
	GetClientRect(&clientRect);
	int windowSize;
	if(bar==SB_HORZ) {
		windowSize=Width(clientRect);
	} else {
		windowSize=Height(clientRect);
	}

	if(newPos<0) {
		newPos=0;
	} else if(newPos>scrollMax-windowSize) {
		if(scrollMax-windowSize>0) {
			newPos=scrollMax-windowSize;
		} else {
			newPos=0;
		}
	}

	int dx,dy;
	if(bar==SB_HORZ) {
		dx=oldPos-newPos;
		dy=0;
	} else {
		dx=0;
		dy=oldPos-newPos;
	}


	ScrollWindowEx(dx,dy,&clientRect,NULL,NULL,NULL,SW_INVALIDATE|SW_ERASE);

	SCROLLINFO newScrollInfo;
	newScrollInfo.cbSize=sizeof(SCROLLINFO);
	newScrollInfo.fMask=SIF_POS;
	newScrollInfo.nPos=newPos;
	SetScrollInfo(bar,&newScrollInfo,TRUE);
	UpdateWindow();
}

//r[̑傫ύXꂽ
void View::OnSize(UINT,int,int)
{
	RECT clientRect;
	GetClientRect(&clientRect);

	OnSizeSub(SB_HORZ,Width(clientRect),m_scrollSize.cx);
	OnSizeSub(SB_VERT,Height(clientRect),m_scrollSize.cy);
}

//OnSizẻ
void View::OnSizeSub(int bar,int clientSize,int scrollSize)
{
	SCROLLINFO info;
	info.cbSize=sizeof(SCROLLINFO);
	info.fMask=SIF_POS;
	GetScrollInfo(bar,&info);

	if(info.nPos+clientSize>scrollSize) {
		info.nPos=scrollSize-clientSize;
	}
	info.nMin=0;
	info.nMax=scrollSize;
	info.nPage=clientSize;
	info.fMask=SIF_POS|SIF_RANGE|SIF_PAGE;
	SetScrollInfo(bar,&info,TRUE);
}

//r[̍̍W𓾂
POINT View::GetScrollPos() const
{
	RECT clientRect;
	GetClientRect(&clientRect);

	POINT point;
	point.x=GetScrollPosSub(SB_HORZ,Width(clientRect),m_scrollSize.cx);
	point.y=GetScrollPosSub(SB_VERT,Height(clientRect),m_scrollSize.cy);
	return point;
}

//GetScrollPos̉
int View::GetScrollPosSub(int bar,int clientSize,int scrollSize) const
{
	int pos;
	if(clientSize>=scrollSize) {
		pos=0;
	} else {
		SCROLLINFO scrollInfo;
		scrollInfo.cbSize=sizeof(SCROLLINFO);
		scrollInfo.fMask=SIF_POS;
		GetScrollInfo(bar,&scrollInfo);
		pos=scrollInfo.nPos;
	}
	return pos;
}

//r[̍Wݒ
void View::OnPrepareDC(HDC hDC)
{
	POINT scroll=GetScrollPos();
	::SetViewportOrgEx(hDC,-scroll.x,-scroll.y,NULL);
}

//ŏ̏
void View::OnInitialUpdate()
{
}

//Xe[^Xo[͂̕̕\
void View::SetStatusText(int textID)
{
	GetFrame()->SetStatusText(textID);
}

void View::OnUpdateCommand(int,Command *)
{

}

//hLg
void View::OnFileClose()
{
	GetDocument()->OnCloseDocument();
}

//ĂȂƂ
bool View::OnIdle()
{
	return false;
}

APP_END_NAMESPACE
