#include "prefix.h"

#include "volumecontrol.h"
#include "winmessage.h"
#include "dc.h"
#include "gdiobject.h"
#include "string.h"

#include "resstring.h"

#include <windows.h>
#include <assert.h>

/*************************************************

	ʃRg[̃NX

**************************************************/

APP_BEGIN_NAMESPACE

VolumeControl::VolumeControl()
:m_volume(0)
,m_upperLimit(1)
,m_lowerLimit(0)
,m_showMode(VOLUME_SHOW_FLOAT)
{
	
}

VolumeControl::~VolumeControl() throw()
{
}

LRESULT VolumeControl::WindowProc(UINT msg,WPARAM wparam,LPARAM lparam)
{
	switch(msg) {
	case WM_LBUTTONDOWN:
		OnLButtonDown((short)LOWORD(lparam));
		break;

	case WM_MOUSEMOVE:
		OnMouseMove((short)LOWORD(lparam));
		break;

	case WM_LBUTTONUP:
		OnLButtonUp();
		break;

	case WM_GETDLGCODE:
		return OnGetDlgCode();

	case APP_WM_REFLECT_DRAWITEM:
		OnDrawItem((DRAWITEMSTRUCT*)wparam);
		break;

	default:
		return Window::WindowProc(msg,wparam,lparam);
	}
	return 0;
}

//ʂ\F
#define BARCOLOR	0xffff00

//\
void VolumeControl::OnDrawItem(DRAWITEMSTRUCT *drawItemStruct)
{
	DrawItemDC dc(drawItemStruct->hDC);
	RECT rect=drawItemStruct->rcItem;

	bool isDisabled=(drawItemStruct->itemState & ODS_DISABLED)!=0;

	int backColor=COLOR_WINDOW;
	if(isDisabled) {
		backColor=COLOR_3DFACE;
	}

	GdiObject frameBrush(::GetSysColorBrush(backColor));
	dc.SelectObject(&frameBrush);
	VERIFY(::Rectangle(dc.Get(),rect.left,rect.top,rect.right,rect.bottom));

	RECT innerRect=rect;
	::InflateRect(&innerRect,-1,-1);
	if(!isDisabled) {
		RECT barRect=innerRect;

		assert(m_lowerLimit<=m_upperLimit);
		double volume=m_volume;
		if(volume<m_lowerLimit) volume=m_lowerLimit;
		if(volume>m_upperLimit) volume=m_upperLimit;
		double volumeRate=0;
		if(m_upperLimit-m_lowerLimit>0) {
			volumeRate=(volume-m_lowerLimit)/(m_upperLimit-m_lowerLimit);
		}
	
		int width=Width(barRect);
		barRect.right=barRect.left+static_cast<int>
			(width*volumeRate);

		GdiObject barBrush(::CreateSolidBrush(BARCOLOR));
		VERIFY(::FillRect(dc.Get(),&barRect,barBrush.GetBrush()));
	}

	dc.SetBkMode(TRANSPARENT);

	RECT textRect=innerRect;
	::InflateRect(&textRect,-1,0);

	String title;
	GetWindowText(&title);
	VERIFY(::DrawText(dc.Get(),title.Get(),title.Length()
		,&textRect,DT_LEFT|DT_SINGLELINE|DT_VCENTER)>0);

	if(!isDisabled) {
		if(m_showMode==VOLUME_SHOW_OWNER) {
			DRAWVOLUME drawVolume;
			drawVolume.hdc=dc.Get();
			drawVolume.rect=textRect;
			drawVolume.volume=m_volume;
			::SendMessage(GetParent(),APP_WM_CONTROL
				,MAKELONG(GetID(),VOLUME_DRAW_VALUE)
				,(LPARAM)&drawVolume);
		} else {
			int format;
			double value=m_volume;
			switch(m_showMode) {
			case VOLUME_SHOW_FLOAT:
				format=IDS_FORMAT_FLOAT;
				break;

			case VOLUME_SHOW_PERCENT:
				format=IDS_FORMAT_PERCENT;
				value*=100;
				break;

			case VOLUME_SHOW_INTEGER:
				format=IDS_FORMAT_INTEGER;
				break;

			case VOLUME_SHOW_PIXEL:
				format=IDS_FORMAT_PIXELS;
				break;

			case VOLUME_SHOW_FLOAT_PIXEL:
				format=IDS_FORMAT_FLOAT_PIXELS;
				break;

			case VOLUME_SHOW_DEGREES:
				format=IDS_FORMAT_DEGREES;
				break;

			default:
				assert(false);
			}

			String string;
			string.Format(format,value);
			VERIFY(::DrawText(dc.Get(),string.Get(),string.Length()
				,&(textRect),DT_RIGHT|DT_SINGLELINE|DT_VCENTER)>0);
		}
	}
}

//󂯕tL[͂̎
UINT VolumeControl::OnGetDlgCode() const
{
	return 0;
}

//{^
void VolumeControl::OnLButtonDown(int x)
{
	Change(x);
	SetCapture();
}

//}EX𓮂
void VolumeControl::OnMouseMove(int x)
{
	if(::GetCapture()==GetHWND()) {
		Change(x);
	}
}

//{^𗣂
void VolumeControl::OnLButtonUp()
{
	if(::GetCapture()==GetHWND()) {
		::ReleaseCapture();
		::PostMessage(GetParent(),APP_WM_CONTROL
			,MAKELONG(GetID(),VOLUME_CHANGED),0);
	}
}

//ʂύX
void VolumeControl::Change(int x)
{
	RECT rect;
	GetClientRect(&rect);
	int width=Width(rect);

	double volumeRate=static_cast<double>(x-rect.left)
		/width;

	m_volume=volumeRate*(m_upperLimit-m_lowerLimit)+m_lowerLimit;

	if(m_volume<m_lowerLimit) {
		m_volume=m_lowerLimit;
	}
	if(m_volume>m_upperLimit) {
		m_volume=m_upperLimit;
	}

	InvalidateRect();
}

//݂̗ʂ𓾂
double VolumeControl::GetVolume() const
{
	return m_volume;
}

//ʂݒ肷
void VolumeControl::SetVolume(double volume)
{
	m_volume=volume;
	InvalidateRect();
}

//E̐ݒ
void VolumeControl::SetRange(double lowerLimit,double upperLimit)
{
	assert(lowerLimit<=upperLimit);
	m_lowerLimit=lowerLimit;
	m_upperLimit=upperLimit;
}

//\@̐ݒ
void VolumeControl::SetShowMode(int showMode)
{
	assert(0<=showMode && showMode<VOLUME_SHOWMODE_MAX);
	m_showMode=showMode;
}

//EBhEjƂ
void VolumeControl::OnDestroy()
{
	UnsubclassWindow();
	Window::OnDestroy();
}

APP_END_NAMESPACE