#include "prefix.h"

#include "window.h"
#include "main.h"
#include "resmain.h"
#include "exception.h"
#include "error.h"
#include "string.h"
#include "verify.h"
#include "showerror.h"

#include <windows.h>
#include <assert.h>
#include <exception>
#include <tchar.h>

/*************************************************

	EBhENX

**************************************************/

APP_BEGIN_NAMESPACE

//EBhEnhɊtꂽIuWFNg̃|C^𓾂
Window *GetWindowObject(HWND hwnd)
{
	return reinterpret_cast<Window*>
		(::GetWindowLong(hwnd,GWL_USERDATA));
}

//EChEo^
static const _TCHAR * s_windowClassName=_T("AppWindow");

LRESULT CALLBACK PreAttachWindowProc(HWND,UINT,WPARAM,LPARAM);
LRESULT CALLBACK AppWindowProc(HWND,UINT,WPARAM,LPARAM);

//IuWFNgɊ֘AtÕbZ[W
LRESULT CALLBACK PreAttachWindowProc(HWND hwnd,UINT msg,WPARAM wparam,LPARAM lparam)
{
	return ::DefWindowProc(hwnd,msg,wparam,lparam);
}

//IuWFNgɊ֘At̃bZ[W
LRESULT CALLBACK AppWindowProc(HWND hwnd,UINT msg,WPARAM wparam,LPARAM lparam)
{
	Window *window=(Window*)::GetWindowLong(hwnd,GWL_USERDATA);
	LRESULT result=0;
	try {
		result=window->WindowProc(msg,wparam,lparam);
	} catch(std::exception &e) {
		ShowError(NULL,e);
	}
	return result;
}

Window::Window()
:m_oldProc(NULL)
,m_hwnd(NULL)
{
}

Window::~Window() throw()
{
	if(IsWindow()) {
		Window::DestroyWindow();
	}
}

//APĨEBhENX̓o^
void Window::RegisterWindowClass()
{
	static bool registered=false;
	if(registered) return;

	registered=true;;

	WNDCLASS wc;

	wc.lpszClassName=s_windowClassName;
	wc.lpfnWndProc=(WNDPROC)PreAttachWindowProc;
	wc.style=CS_OWNDC|CS_VREDRAW|CS_HREDRAW;

	wc.hInstance=GetInstance();

	wc.hIcon=::LoadIcon(GetInstance(),MAKEINTRESOURCE(IDR_MAIN));
	assert(wc.hIcon!=NULL);
	wc.hCursor=::LoadCursor(NULL,IDC_ARROW);
	assert(wc.hCursor!=NULL);
	wc.hbrBackground=reinterpret_cast<HBRUSH>(COLOR_APPWORKSPACE+1);
	wc.lpszMenuName=MAKEINTRESOURCE(IDR_MAIN);
	wc.cbClsExtra=0;
	wc.cbWndExtra=0;

	VERIFY(::RegisterClass(&wc));
}

//EBhE쐬
void Window::Create(const _TCHAR * caption,UINT style,const RECT &rect,HWND parentWindow)
{
	RegisterWindowClass();

	int width=Width(rect);
	if(rect.right==CW_USEDEFAULT) {
		width=CW_USEDEFAULT;
	} else if(rect.left==CW_USEDEFAULT) {
		width=rect.right;
	}
	int height=Height(rect);
	if(rect.bottom==CW_USEDEFAULT) {
		height=CW_USEDEFAULT;
	} else if(rect.top==CW_USEDEFAULT) {
		height=rect.bottom;
	}

	HWND hwnd=::CreateWindow(s_windowClassName
	,caption
	,style
	,rect.left,rect.top,width,height
	,parentWindow
	,NULL
	,GetInstance()
	,NULL);
	if(hwnd==NULL) {
		throw Exception(APPERR_CANT_CREATE_WINDOW);
	}

	SubclassWindow(hwnd);
}

//EBhEIuWFNgɊt
void Window::SubclassWindow(HWND hwnd)
{
	assert(hwnd!=NULL && m_hwnd==NULL);
	m_hwnd=hwnd;
	m_oldProc=reinterpret_cast<WNDPROC>
		(GetWindowLong(GWL_WNDPROC));
	assert(m_oldProc!=NULL);
	SetWindowLong(GWL_USERDATA,(LONG)this);
	SetWindowLong(GWL_WNDPROC,(LONG)AppWindowProc);
}

//TuNXEBhEɖ߂
void Window::UnsubclassWindow()
{
	assert(m_hwnd!=NULL);
	SetWindowLong(GWL_USERDATA,NULL);
	SetWindowLong(GWL_WNDPROC,(LONG)m_oldProc);
	m_hwnd=NULL;
}

//bZ[WU蕪
LRESULT Window::WindowProc(UINT msg,WPARAM wparam,LPARAM lparam)
{
	switch(msg) {
	case WM_COMMAND:
		OnCommand(LOWORD(wparam),HIWORD(wparam),(HWND)lparam);
		break;

	case WM_DESTROY:
		OnDestroy();
		break;

	default:
		return OldProc(msg,wparam,lparam);
	}
	return 0;
}

//TuNXÕbZ[Wɏn
LRESULT Window::OldProc(UINT msg,WPARAM wparam,LPARAM lparam)
{
	return (*m_oldProc)(GetHWND(),msg,wparam,lparam);
}

bool Window::ShowWindow(int mode)
{
	return ::ShowWindow(GetHWND(),mode)!=FALSE;
}

bool Window::DestroyWindow()
{
	return ::DestroyWindow(GetHWND())!=FALSE;
}

bool Window::GetClientRect(RECT *rect) const
{
	return ::GetClientRect(GetHWND(),rect)!=FALSE;
}

bool Window::GetWindowRect(RECT *rect) const
{
	return ::GetWindowRect(GetHWND(),rect)!=FALSE;
}

bool Window::SetWindowPos(Window *wndInsertAfter,int x,int y,int cx,int cy,UINT flags)
{
	return ::SetWindowPos(GetHWND(),wndInsertAfter->GetHWND(),x,y,cx,cy,flags)!=FALSE;
}

LONG Window::SetWindowLong(int index,LONG dwNewLong)
{
	return ::SetWindowLong(GetHWND(),index,dwNewLong);
}

LONG Window::GetWindowLong(int index) const
{
	return ::GetWindowLong(GetHWND(),index);
}

//EBhEnh𓾂
HWND Window::GetHWND() const
{
	if(this==NULL) {
		return NULL;
	}
	return m_hwnd;
}

//R}h
void Window::OnCommand(int,int,HWND)
{
}

//EBhEj
void Window::OnDestroy()
{
	m_hwnd=NULL;
}

bool Window::SetScrollInfo(int fnBar,SCROLLINFO *pSi,BOOL redraw)
{
	return ::SetScrollInfo(GetHWND(),fnBar,pSi,redraw)!=FALSE;
}

bool Window::GetScrollInfo(int fnBar,SCROLLINFO *pSi) const
{
	return ::GetScrollInfo(GetHWND(),fnBar,pSi)!=FALSE;
}

int Window::ScrollWindowEx(int dx,int dy,const RECT *scroll
		,const RECT *clip,HRGN updateRegion
		,RECT *updateRect,UINT flags)
{
	return ::ScrollWindowEx(GetHWND(),dx,dy,scroll
		,clip,updateRegion,updateRect,flags);
}

bool Window::UpdateWindow()
{
	return ::UpdateWindow(GetHWND())!=FALSE;
}

LRESULT Window::SendMessage(UINT message,WPARAM wparam,LPARAM lparam)
{
	return ::SendMessage(GetHWND(),message,wparam,lparam);
}

bool Window::PostMessage(UINT message,WPARAM wparam,LPARAM lparam)
{
	return ::PostMessage(GetHWND(),message,wparam,lparam)!=FALSE;
}

bool Window::InvalidateRect(const RECT *rect,bool doesErase)
{
	return ::InvalidateRect(GetHWND(),rect,doesErase?TRUE:FALSE)!=FALSE;
}

bool Window::ScreenToClient(POINT *point) const
{
	return ::ScreenToClient(GetHWND(),point)!=FALSE;
}

bool Window::ClientToScreen(POINT *point) const
{
	return ::ClientToScreen(GetHWND(),point)!=FALSE;
}

HWND Window::SetCapture()
{
	return ::SetCapture(GetHWND());
}

bool Window::DragDetect(POINT point) const
{
	return ::DragDetect(GetHWND(),point)!=FALSE;
}

bool Window::SetWindowText(const _TCHAR * text)
{
	return ::SetWindowText(GetHWND(),text)!=FALSE;
}

void Window::GetWindowText(String *string) const
{
	int length=::GetWindowTextLength(GetHWND())+1;
	string->Alloc(length);
	::GetWindowText(GetHWND(),string->GetW(),length);
	string->Shorten();
}

void Window::Attach(HWND hwnd)
{
	m_hwnd=hwnd;
}

void Window::Detach()
{
	assert(m_hwnd!=NULL);
	assert(m_oldProc==NULL);
	m_hwnd=NULL;
}

bool Window::IsWindow() const throw()
{
	return ::IsWindow(GetHWND())!=FALSE;
}

bool Window::IsWindowEnabled() const
{
	return ::IsWindowEnabled(GetHWND())!=FALSE;
}

bool Window::IsWindowVisible() const
{
	return ::IsWindowVisible(GetHWND())!=FALSE;
}

HWND Window::GetDlgItem(int itemID) const
{
	return ::GetDlgItem(GetHWND(),itemID);
}

bool Window::SetDlgItemText(int itemID,const _TCHAR * text)
{
	assert(GetDlgItem(itemID)!=NULL);

	String curString;
	GetDlgItemText(itemID,&curString);

	if(curString.Compare(text)!=0) {
		return ::SetDlgItemText(GetHWND(),itemID,text)!=FALSE;
	}
	return false;
}

void Window::GetDlgItemText(int itemID,String *string) const
{
	HWND item=GetDlgItem(itemID);
	assert(item!=NULL);

	int length=::GetWindowTextLength(item)+1;
	string->Alloc(length);
	::GetWindowText(item,string->GetW(),length);
	string->Shorten();
}

bool Window::SetDlgItemInt(int itemID,int value)
{
	assert(GetDlgItem(itemID)!=NULL);
	BOOL succeed;
	int curValue=GetDlgItemInt(itemID,&succeed);

	if(!succeed || curValue!=value) {
		return ::SetDlgItemInt(GetHWND(),itemID
			,value,TRUE)!=FALSE;
	}
	return false;
}

int Window::GetDlgItemInt(int itemID,BOOL *succeed) const
{
	assert(GetDlgItem(itemID)!=NULL);
	return  ::GetDlgItemInt(GetHWND(),itemID
		,succeed,TRUE);
}

bool Window::CheckDlgButton(int itemID,UINT state)
{
	assert(GetDlgItem(itemID)!=NULL);
	return ::CheckDlgButton(GetHWND(),itemID,state)!=FALSE;
}

UINT Window::IsDlgButtonChecked(int itemID) const
{
	assert(GetDlgItem(itemID)!=NULL);
	return ::IsDlgButtonChecked(GetHWND(),itemID);
}

void Window::SubclassDialogItem(UINT itemID,Window *parent)
{
	HWND item=parent->GetDlgItem(itemID);
	assert(item!=NULL);
	SubclassWindow(item);
}

LONG Window::SendDlgItemMessage(int itemID,UINT msg,WPARAM wparam,LPARAM lparam)
{
	assert(GetDlgItem(itemID)!=NULL);
	return ::SendDlgItemMessage(GetHWND(),itemID,msg,wparam,lparam);
}

LONG Window::GetID() const
{
	return GetWindowLong(GWL_ID);
}

HWND Window::GetParent() const
{
	return ::GetParent(GetHWND());
}

APP_END_NAMESPACE



