#include "prefix.h"

#include "winman.h"

#include "appview.h"
#include "appdoc.h"
#include "appframe.h"

#include "winmessage.h"
#include "registry.h"

#include "panel.h"
#include "generalpanel.h"
#include "spraypanel.h"
#include "colorpanel.h"
#include "catalogpanel.h"

#include <windows.h>

/*************************************************

	EBhE̊Ǘ

**************************************************/

APP_BEGIN_NAMESPACE

static AppFrame *s_frame=NULL;
HWND s_disabledWindow=NULL;
Panel *s_panels[PANEL_COUNT];

static void CreateFirstWindow(HWND parentWindow);
static void CreateAllPanels(HWND frameWindow);

//AvP[V̏
void AppInit_WinMan(Registry *registry,HWND parentWindow)
{
	if(parentWindow!=NULL && ::IsWindowEnabled(parentWindow)) {
		::EnableWindow(parentWindow,FALSE);
		s_disabledWindow=parentWindow;
	}

	FOR(int i=0;i<PANEL_COUNT;++i) {
		s_panels[i]=NULL;
	}

	s_panels[PANEL_GENERAL]=new GeneralPanel;
	s_panels[PANEL_SPRAY]=new SprayPanel;
	s_panels[PANEL_COLOR]=new ColorPanel;
	s_panels[PANEL_CATALOG]=new CatalogPanel;

	FOR(int i=0;i<PANEL_COUNT;++i) {
		s_panels[i]->AppInit(registry);
	}

	s_frame=new AppFrame;
	s_frame->AppInit(registry);
	CreateFirstWindow(parentWindow);
	CreateAllPanels(s_frame->GetHWND());

	s_frame->SendMessage(APP_WM_UPDATE_WINDOW,0,0);
}

//ݒ̕ۑ
void AppSave_WinMan(Registry *registry)
{
	FOR(int i=0;i<PANEL_COUNT;++i) {
		s_panels[i]->AppSave(registry);
	}
	s_frame->AppSave(registry);
}

//AvP[V̏I
void AppQuit_WinMan() throw()
{
	if(s_disabledWindow!=NULL) {
		::EnableWindow(s_disabledWindow,TRUE);
	}

	FOR(int i=0;i<PANEL_COUNT;++i) {
		if(s_panels[i]!=NULL) {
			s_panels[i]->AppQuit();
			delete s_panels[i];
		}
	}
	if(s_frame!=NULL) {
		s_frame->AppQuit();
		delete s_frame;
	}
}

//CEBhE
static void CreateFirstWindow(HWND parentWindow)
{
	assert(s_frame!=NULL);

	AppView *view=new AppView;
	s_frame->AttachView(view);
	view->SetFrame(s_frame);

	AppDoc *document=new AppDoc;
	view->AttachDocument(document);

	s_frame->Create(parentWindow);
}

//Ct[EBhEꂽ
void ClosedMainFrame(Frame *frameWindow)
{
	assert(frameWindow==s_frame);
	::PostQuitMessage(0);
}

//SẴEBhEɃbZ[W𑗂
void SendAllWindowsMessage(UINT msg,WPARAM wparam,LPARAM lparam)
{
	s_frame->SendMessage(msg,wparam,lparam);
	s_frame->GetView()->SendMessage(msg,wparam,lparam);

	SendAllPanelsMessage(msg,wparam,lparam);
}

//CEBhẼnh𓾂
HWND GetFrameHWND()
{
	if(s_frame==NULL) {
		return NULL;
	}
	HWND hwnd=s_frame->GetHWND();
	if(!::IsWindow(hwnd)) {
		hwnd=NULL;
	}
	return hwnd;
}

//r[̃nh𓾂
HWND GetViewHWND()
{
	if(s_frame==NULL) {
		return NULL;
	}
	if(s_frame->GetView()==NULL) {
		return NULL;
	}
	HWND hwnd=s_frame->GetView()->GetHWND();
	if(!::IsWindow(hwnd)) {
		hwnd=NULL;
	}
	return hwnd;
}

//SẴplɃbZ[W𑗂
void SendAllPanelsMessage(UINT msg,WPARAM wparam,LPARAM lparam)
{
	FOR(int i=0;i<PANEL_COUNT;++i) {
		SendPanelMessage(i,msg,wparam,lparam);
	}
}

//plɃbZ[W𑗂
void SendPanelMessage(int panelIndex,UINT msg,WPARAM wparam,LPARAM lparam)
{
	assert(0<=panelIndex && panelIndex<PANEL_COUNT);
	Panel *panel=s_panels[panelIndex];
	assert(panel!=NULL);
	if(panel->IsWindow()) {
		panel->SendMessage(msg,wparam,lparam);
	}
}

//SẴpl
static void CreateAllPanels(HWND frameWindow)
{
	FOR(int i=0;i<PANEL_COUNT;++i) {
		Panel *panel=s_panels[i];
		assert(panel!=NULL);
		panel->Create(frameWindow);
	}
}

//pl\E\
void ToggleShowPanel(int panelType)
{
	assert(0<=panelType && panelType<PANEL_COUNT);

	Panel *panel=s_panels[panelType];
	assert(panel!=NULL);

	int showMode;
	if(panel->IsWindowVisible()) {
		showMode=SW_HIDE;
	} else {
		showMode=SW_SHOW;
	}
	panel->ShowWindow(showMode);
}

//pl\Ă邩
bool IsPanelVisible(int panelType)
{
	assert(0<=panelType && panelType<PANEL_COUNT);

	Panel *panel=s_panels[panelType];
	assert(panel!=NULL);
	return panel->IsWindowVisible();
}

//ĂȂƂ
bool OnIdle_WinMan()
{
	if(s_frame!=NULL) {
		return s_frame->OnIdle();
	}
	return false;
}

APP_END_NAMESPACE
